# -*- coding: utf-8 -*-
# Copyright 2024 Google LLC
#
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
#     http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.
#
from __future__ import annotations

from typing import MutableMapping, MutableSequence

import proto  # type: ignore

__protobuf__ = proto.module(
    package="google.cloud.automl.v1beta1",
    manifest={
        "InputConfig",
        "BatchPredictInputConfig",
        "DocumentInputConfig",
        "OutputConfig",
        "BatchPredictOutputConfig",
        "ModelExportOutputConfig",
        "ExportEvaluatedExamplesOutputConfig",
        "GcsSource",
        "BigQuerySource",
        "GcsDestination",
        "BigQueryDestination",
        "GcrDestination",
    },
)


class InputConfig(proto.Message):
    r"""Input configuration for ImportData Action.

    The format of input depends on dataset_metadata the Dataset into
    which the import is happening has. As input source the
    [gcs_source][google.cloud.automl.v1beta1.InputConfig.gcs_source] is
    expected, unless specified otherwise. Additionally any input .CSV
    file by itself must be 100MB or smaller, unless specified otherwise.
    If an "example" file (that is, image, video etc.) with identical
    content (even if it had different GCS_FILE_PATH) is mentioned
    multiple times, then its label, bounding boxes etc. are appended.
    The same file should be always provided with the same ML_USE and
    GCS_FILE_PATH, if it is not, then these values are
    nondeterministically selected from the given ones.

    The formats are represented in EBNF with commas being literal and
    with non-terminal symbols defined near the end of this comment. The
    formats are:

    -  For Image Classification: CSV file(s) with each line in format:
       ML_USE,GCS_FILE_PATH,LABEL,LABEL,... GCS_FILE_PATH leads to image
       of up to 30MB in size. Supported extensions: .JPEG, .GIF, .PNG,
       .WEBP, .BMP, .TIFF, .ICO For MULTICLASS classification type, at
       most one LABEL is allowed per image. If an image has not yet been
       labeled, then it should be mentioned just once with no LABEL.
       Some sample rows: TRAIN,gs://folder/image1.jpg,daisy
       TEST,gs://folder/image2.jpg,dandelion,tulip,rose
       UNASSIGNED,gs://folder/image3.jpg,daisy
       UNASSIGNED,gs://folder/image4.jpg

    -  For Image Object Detection: CSV file(s) with each line in format:
       ML_USE,GCS_FILE_PATH,(LABEL,BOUNDING_BOX \| ,,,,,,,)
       GCS_FILE_PATH leads to image of up to 30MB in size. Supported
       extensions: .JPEG, .GIF, .PNG. Each image is assumed to be
       exhaustively labeled. The minimum allowed BOUNDING_BOX edge
       length is 0.01, and no more than 500 BOUNDING_BOX-es per image
       are allowed (one BOUNDING_BOX is defined per line). If an image
       has not yet been labeled, then it should be mentioned just once
       with no LABEL and the ",,,,,,," in place of the BOUNDING_BOX. For
       images which are known to not contain any bounding boxes, they
       should be labelled explictly as "NEGATIVE_IMAGE", followed by
       ",,,,,,," in place of the BOUNDING_BOX. Sample rows:
       TRAIN,gs://folder/image1.png,car,0.1,0.1,,,0.3,0.3,,
       TRAIN,gs://folder/image1.png,bike,.7,.6,,,.8,.9,,
       UNASSIGNED,gs://folder/im2.png,car,0.1,0.1,0.2,0.1,0.2,0.3,0.1,0.3
       TEST,gs://folder/im3.png,,,,,,,,,
       TRAIN,gs://folder/im4.png,NEGATIVE_IMAGE,,,,,,,,,

    -  For Video Classification: CSV file(s) with each line in format:
       ML_USE,GCS_FILE_PATH where ML_USE VALIDATE value should not be
       used. The GCS_FILE_PATH should lead to another .csv file which
       describes examples that have given ML_USE, using the following
       row format:
       GCS_FILE_PATH,(LABEL,TIME_SEGMENT_START,TIME_SEGMENT_END \| ,,)
       Here GCS_FILE_PATH leads to a video of up to 50GB in size and up
       to 3h duration. Supported extensions: .MOV, .MPEG4, .MP4, .AVI.
       TIME_SEGMENT_START and TIME_SEGMENT_END must be within the length
       of the video, and end has to be after the start. Any segment of a
       video which has one or more labels on it, is considered a hard
       negative for all other labels. Any segment with no labels on it
       is considered to be unknown. If a whole video is unknown, then it
       shuold be mentioned just once with ",," in place of LABEL,
       TIME_SEGMENT_START,TIME_SEGMENT_END. Sample top level CSV file:
       TRAIN,gs://folder/train_videos.csv
       TEST,gs://folder/test_videos.csv
       UNASSIGNED,gs://folder/other_videos.csv Sample rows of a CSV file
       for a particular ML_USE:
       gs://folder/video1.avi,car,120,180.000021
       gs://folder/video1.avi,bike,150,180.000021
       gs://folder/vid2.avi,car,0,60.5 gs://folder/vid3.avi,,,

    -  For Video Object Tracking: CSV file(s) with each line in format:
       ML_USE,GCS_FILE_PATH where ML_USE VALIDATE value should not be
       used. The GCS_FILE_PATH should lead to another .csv file which
       describes examples that have given ML_USE, using one of the
       following row format:
       GCS_FILE_PATH,LABEL,[INSTANCE_ID],TIMESTAMP,BOUNDING_BOX or
       GCS_FILE_PATH,,,,,,,,,, Here GCS_FILE_PATH leads to a video of up
       to 50GB in size and up to 3h duration. Supported extensions:
       .MOV, .MPEG4, .MP4, .AVI. Providing INSTANCE_IDs can help to
       obtain a better model. When a specific labeled entity leaves the
       video frame, and shows up afterwards it is not required, albeit
       preferable, that the same INSTANCE_ID is given to it. TIMESTAMP
       must be within the length of the video, the BOUNDING_BOX is
       assumed to be drawn on the closest video's frame to the
       TIMESTAMP. Any mentioned by the TIMESTAMP frame is expected to be
       exhaustively labeled and no more than 500 BOUNDING_BOX-es per
       frame are allowed. If a whole video is unknown, then it should be
       mentioned just once with ",,,,,,,,,," in place of LABEL,
       [INSTANCE_ID],TIMESTAMP,BOUNDING_BOX. Sample top level CSV file:
       TRAIN,gs://folder/train_videos.csv
       TEST,gs://folder/test_videos.csv
       UNASSIGNED,gs://folder/other_videos.csv Seven sample rows of a
       CSV file for a particular ML_USE:
       gs://folder/video1.avi,car,1,12.10,0.8,0.8,0.9,0.8,0.9,0.9,0.8,0.9
       gs://folder/video1.avi,car,1,12.90,0.4,0.8,0.5,0.8,0.5,0.9,0.4,0.9
       gs://folder/video1.avi,car,2,12.10,.4,.2,.5,.2,.5,.3,.4,.3
       gs://folder/video1.avi,car,2,12.90,.8,.2,,,.9,.3,,
       gs://folder/video1.avi,bike,,12.50,.45,.45,,,.55,.55,,
       gs://folder/video2.avi,car,1,0,.1,.9,,,.9,.1,,
       gs://folder/video2.avi,,,,,,,,,,,

    -  For Text Extraction: CSV file(s) with each line in format:
       ML_USE,GCS_FILE_PATH GCS_FILE_PATH leads to a .JSONL (that is,
       JSON Lines) file which either imports text in-line or as
       documents. Any given .JSONL file must be 100MB or smaller. The
       in-line .JSONL file contains, per line, a proto that wraps a
       TextSnippet proto (in json representation) followed by one or
       more AnnotationPayload protos (called annotations), which have
       display_name and text_extraction detail populated. The given text
       is expected to be annotated exhaustively, for example, if you
       look for animals and text contains "dolphin" that is not labeled,
       then "dolphin" is assumed to not be an animal. Any given text
       snippet content must be 10KB or smaller, and also be UTF-8 NFC
       encoded (ASCII already is). The document .JSONL file contains,
       per line, a proto that wraps a Document proto. The Document proto
       must have either document_text or input_config set. In
       document_text case, the Document proto may also contain the
       spatial information of the document, including layout, document
       dimension and page number. In input_config case, only PDF
       documents are supported now, and each document may be up to 2MB
       large. Currently, annotations on documents cannot be specified at
       import. Three sample CSV rows: TRAIN,gs://folder/file1.jsonl
       VALIDATE,gs://folder/file2.jsonl TEST,gs://folder/file3.jsonl
       Sample in-line JSON Lines file for entity extraction (presented
       here with artificial line breaks, but the only actual line break
       is denoted by \\n).: { "document": { "document_text": {"content":
       "dog cat"} "layout": [ { "text_segment": { "start_offset": 0,
       "end_offset": 3, }, "page_number": 1, "bounding_poly": {
       "normalized_vertices": [ {"x": 0.1, "y": 0.1}, {"x": 0.1, "y":
       0.3}, {"x": 0.3, "y": 0.3}, {"x": 0.3, "y": 0.1}, ], },
       "text_segment_type": TOKEN, }, { "text_segment": {
       "start_offset": 4, "end_offset": 7, }, "page_number": 1,
       "bounding_poly": { "normalized_vertices": [ {"x": 0.4, "y": 0.1},
       {"x": 0.4, "y": 0.3}, {"x": 0.8, "y": 0.3}, {"x": 0.8, "y": 0.1},
       ], }, "text_segment_type": TOKEN, }

       ::

                ],
                "document_dimensions": {
                  "width": 8.27,
                  "height": 11.69,
                  "unit": INCH,
                }
                "page_count": 1,
              },
              "annotations": [
                {
                  "display_name": "animal",
                  "text_extraction": {"text_segment": {"start_offset": 0,
                  "end_offset": 3}}
                },
                {
                  "display_name": "animal",
                  "text_extraction": {"text_segment": {"start_offset": 4,
                  "end_offset": 7}}
                }
              ],
            }\n
            {
               "text_snippet": {
                 "content": "This dog is good."
               },
               "annotations": [
                 {
                   "display_name": "animal",
                   "text_extraction": {
                     "text_segment": {"start_offset": 5, "end_offset": 8}
                   }
                 }
               ]
            }
          Sample document JSON Lines file (presented here with artificial line
          breaks, but the only actual line break is denoted by \n).:
            {
              "document": {
                "input_config": {
                  "gcs_source": { "input_uris": [ "gs://folder/document1.pdf" ]
                  }
                }
              }
            }\n
            {
              "document": {
                "input_config": {
                  "gcs_source": { "input_uris": [ "gs://folder/document2.pdf" ]
                  }
                }
              }
            }

    -  For Text Classification: CSV file(s) with each line in format:
       ML_USE,(TEXT_SNIPPET \| GCS_FILE_PATH),LABEL,LABEL,...
       TEXT_SNIPPET and GCS_FILE_PATH are distinguished by a pattern. If
       the column content is a valid gcs file path, i.e. prefixed by
       "gs://", it will be treated as a GCS_FILE_PATH, else if the
       content is enclosed within double quotes (""), it is treated as a
       TEXT_SNIPPET. In the GCS_FILE_PATH case, the path must lead to a
       .txt file with UTF-8 encoding, for example,
       "gs://folder/content.txt", and the content in it is extracted as
       a text snippet. In TEXT_SNIPPET case, the column content
       excluding quotes is treated as to be imported text snippet. In
       both cases, the text snippet/file size must be within 128kB.
       Maximum 100 unique labels are allowed per CSV row. Sample rows:
       TRAIN,"They have bad food and very rude",RudeService,BadFood
       TRAIN,gs://folder/content.txt,SlowService TEST,"Typically always
       bad service there.",RudeService VALIDATE,"Stomach ache to
       go.",BadFood

    -  For Text Sentiment: CSV file(s) with each line in format:
       ML_USE,(TEXT_SNIPPET \| GCS_FILE_PATH),SENTIMENT TEXT_SNIPPET and
       GCS_FILE_PATH are distinguished by a pattern. If the column
       content is a valid gcs file path, that is, prefixed by "gs://",
       it is treated as a GCS_FILE_PATH, otherwise it is treated as a
       TEXT_SNIPPET. In the GCS_FILE_PATH case, the path must lead to a
       .txt file with UTF-8 encoding, for example,
       "gs://folder/content.txt", and the content in it is extracted as
       a text snippet. In TEXT_SNIPPET case, the column content itself
       is treated as to be imported text snippet. In both cases, the
       text snippet must be up to 500 characters long. Sample rows:
       TRAIN,"@freewrytin this is way too good for your product",2
       TRAIN,"I need this product so bad",3 TEST,"Thank you for this
       product.",4 VALIDATE,gs://folder/content.txt,2

    -  For Tables: Either
       [gcs_source][google.cloud.automl.v1beta1.InputConfig.gcs_source]
       or

    [bigquery_source][google.cloud.automl.v1beta1.InputConfig.bigquery_source]
    can be used. All inputs is concatenated into a single

    [primary_table][google.cloud.automl.v1beta1.TablesDatasetMetadata.primary_table_name]
    For gcs_source: CSV file(s), where the first row of the first file
    is the header, containing unique column names. If the first row of a
    subsequent file is the same as the header, then it is also treated
    as a header. All other rows contain values for the corresponding
    columns. Each .CSV file by itself must be 10GB or smaller, and their
    total size must be 100GB or smaller. First three sample rows of a
    CSV file: "Id","First Name","Last Name","Dob","Addresses"

    "1","John","Doe","1968-01-22","[{"status":"current","address":"123_First_Avenue","city":"Seattle","state":"WA","zip":"11111","numberOfYears":"1"},{"status":"previous","address":"456_Main_Street","city":"Portland","state":"OR","zip":"22222","numberOfYears":"5"}]"

    "2","Jane","Doe","1980-10-16","[{"status":"current","address":"789_Any_Avenue","city":"Albany","state":"NY","zip":"33333","numberOfYears":"2"},{"status":"previous","address":"321_Main_Street","city":"Hoboken","state":"NJ","zip":"44444","numberOfYears":"3"}]}
    For bigquery_source: An URI of a BigQuery table. The user data size
    of the BigQuery table must be 100GB or smaller. An imported table
    must have between 2 and 1,000 columns, inclusive, and between 1000
    and 100,000,000 rows, inclusive. There are at most 5 import data
    running in parallel. Definitions: ML_USE = "TRAIN" \| "VALIDATE" \|
    "TEST" \| "UNASSIGNED" Describes how the given example (file) should
    be used for model training. "UNASSIGNED" can be used when user has
    no preference. GCS_FILE_PATH = A path to file on GCS, e.g.
    "gs://folder/image1.png". LABEL = A display name of an object on an
    image, video etc., e.g. "dog". Must be up to 32 characters long and
    can consist only of ASCII Latin letters A-Z and a-z, underscores(_),
    and ASCII digits 0-9. For each label an AnnotationSpec is created
    which display_name becomes the label; AnnotationSpecs are given back
    in predictions. INSTANCE_ID = A positive integer that identifies a
    specific instance of a labeled entity on an example. Used e.g. to
    track two cars on a video while being able to tell apart which one
    is which. BOUNDING_BOX = VERTEX,VERTEX,VERTEX,VERTEX \|
    VERTEX,,,VERTEX,, A rectangle parallel to the frame of the example
    (image, video). If 4 vertices are given they are connected by edges
    in the order provided, if 2 are given they are recognized as
    diagonally opposite vertices of the rectangle. VERTEX =
    COORDINATE,COORDINATE First coordinate is horizontal (x), the second
    is vertical (y). COORDINATE = A float in 0 to 1 range, relative to
    total length of image or video in given dimension. For fractions the
    leading non-decimal 0 can be omitted (i.e. 0.3 = .3). Point 0,0 is
    in top left. TIME_SEGMENT_START = TIME_OFFSET Expresses a beginning,
    inclusive, of a time segment within an example that has a time
    dimension (e.g. video). TIME_SEGMENT_END = TIME_OFFSET Expresses an
    end, exclusive, of a time segment within an example that has a time
    dimension (e.g. video). TIME_OFFSET = A number of seconds as
    measured from the start of an example (e.g. video). Fractions are
    allowed, up to a microsecond precision. "inf" is allowed, and it
    means the end of the example. TEXT_SNIPPET = A content of a text
    snippet, UTF-8 encoded, enclosed within double quotes ("").
    SENTIMENT = An integer between 0 and
    Dataset.text_sentiment_dataset_metadata.sentiment_max (inclusive).
    Describes the ordinal of the sentiment - higher value means a more
    positive sentiment. All the values are completely relative, i.e.
    neither 0 needs to mean a negative or neutral sentiment nor
    sentiment_max needs to mean a positive one - it is just required
    that 0 is the least positive sentiment in the data, and
    sentiment_max is the most positive one. The SENTIMENT shouldn't be
    confused with "score" or "magnitude" from the previous Natural
    Language Sentiment Analysis API. All SENTIMENT values between 0 and
    sentiment_max must be represented in the imported data. On
    prediction the same 0 to sentiment_max range will be used. The
    difference between neighboring sentiment values needs not to be
    uniform, e.g. 1 and 2 may be similar whereas the difference between
    2 and 3 may be huge.

    Errors: If any of the provided CSV files can't be parsed or if more
    than certain percent of CSV rows cannot be processed then the
    operation fails and nothing is imported. Regardless of overall
    success or failure the per-row failures, up to a certain count cap,
    is listed in Operation.metadata.partial_failures.

    This message has `oneof`_ fields (mutually exclusive fields).
    For each oneof, at most one member field can be set at the same time.
    Setting any member of the oneof automatically clears all other
    members.

    .. _oneof: https://proto-plus-python.readthedocs.io/en/stable/fields.html#oneofs-mutually-exclusive-fields

    Attributes:
        gcs_source (google.cloud.automl_v1beta1.types.GcsSource):
            The Google Cloud Storage location for the input content. In
            ImportData, the gcs_source points to a csv with structure
            described in the comment.

            This field is a member of `oneof`_ ``source``.
        bigquery_source (google.cloud.automl_v1beta1.types.BigQuerySource):
            The BigQuery location for the input content.

            This field is a member of `oneof`_ ``source``.
        params (MutableMapping[str, str]):
            Additional domain-specific parameters describing the
            semantic of the imported data, any string must be up to
            25000 characters long.

            -  For Tables: ``schema_inference_version`` - (integer)
               Required. The version of the algorithm that should be
               used for the initial inference of the schema (columns'
               DataTypes) of the table the data is being imported into.
               Allowed values: "1".
    """

    gcs_source: "GcsSource" = proto.Field(
        proto.MESSAGE,
        number=1,
        oneof="source",
        message="GcsSource",
    )
    bigquery_source: "BigQuerySource" = proto.Field(
        proto.MESSAGE,
        number=3,
        oneof="source",
        message="BigQuerySource",
    )
    params: MutableMapping[str, str] = proto.MapField(
        proto.STRING,
        proto.STRING,
        number=2,
    )


class BatchPredictInputConfig(proto.Message):
    r"""Input configuration for BatchPredict Action.

    The format of input depends on the ML problem of the model used for
    prediction. As input source the
    [gcs_source][google.cloud.automl.v1beta1.InputConfig.gcs_source] is
    expected, unless specified otherwise.

    The formats are represented in EBNF with commas being literal and
    with non-terminal symbols defined near the end of this comment. The
    formats are:

    -  For Image Classification: CSV file(s) with each line having just
       a single column: GCS_FILE_PATH which leads to image of up to 30MB
       in size. Supported extensions: .JPEG, .GIF, .PNG. This path is
       treated as the ID in the Batch predict output. Three sample rows:
       gs://folder/image1.jpeg gs://folder/image2.gif
       gs://folder/image3.png

    -  For Image Object Detection: CSV file(s) with each line having
       just a single column: GCS_FILE_PATH which leads to image of up to
       30MB in size. Supported extensions: .JPEG, .GIF, .PNG. This path
       is treated as the ID in the Batch predict output. Three sample
       rows: gs://folder/image1.jpeg gs://folder/image2.gif
       gs://folder/image3.png

    -  For Video Classification: CSV file(s) with each line in format:
       GCS_FILE_PATH,TIME_SEGMENT_START,TIME_SEGMENT_END GCS_FILE_PATH
       leads to video of up to 50GB in size and up to 3h duration.
       Supported extensions: .MOV, .MPEG4, .MP4, .AVI.
       TIME_SEGMENT_START and TIME_SEGMENT_END must be within the length
       of the video, and end has to be after the start. Three sample
       rows: gs://folder/video1.mp4,10,40 gs://folder/video1.mp4,20,60
       gs://folder/vid2.mov,0,inf

    -  For Video Object Tracking: CSV file(s) with each line in format:
       GCS_FILE_PATH,TIME_SEGMENT_START,TIME_SEGMENT_END GCS_FILE_PATH
       leads to video of up to 50GB in size and up to 3h duration.
       Supported extensions: .MOV, .MPEG4, .MP4, .AVI.
       TIME_SEGMENT_START and TIME_SEGMENT_END must be within the length
       of the video, and end has to be after the start. Three sample
       rows: gs://folder/video1.mp4,10,240
       gs://folder/video1.mp4,300,360 gs://folder/vid2.mov,0,inf

    -  For Text Classification: CSV file(s) with each line having just a
       single column: GCS_FILE_PATH \| TEXT_SNIPPET Any given text file
       can have size upto 128kB. Any given text snippet content must
       have 60,000 characters or less. Three sample rows:
       gs://folder/text1.txt "Some text content to predict"
       gs://folder/text3.pdf Supported file extensions: .txt, .pdf

    -  For Text Sentiment: CSV file(s) with each line having just a
       single column: GCS_FILE_PATH \| TEXT_SNIPPET Any given text file
       can have size upto 128kB. Any given text snippet content must
       have 500 characters or less. Three sample rows:
       gs://folder/text1.txt "Some text content to predict"
       gs://folder/text3.pdf Supported file extensions: .txt, .pdf

    -  For Text Extraction .JSONL (i.e. JSON Lines) file(s) which either
       provide text in-line or as documents (for a single BatchPredict
       call only one of the these formats may be used). The in-line
       .JSONL file(s) contain per line a proto that wraps a temporary
       user-assigned TextSnippet ID (string up to 2000 characters long)
       called "id", a TextSnippet proto (in json representation) and
       zero or more TextFeature protos. Any given text snippet content
       must have 30,000 characters or less, and also be UTF-8 NFC
       encoded (ASCII already is). The IDs provided should be unique.
       The document .JSONL file(s) contain, per line, a proto that wraps
       a Document proto with input_config set. Only PDF documents are
       supported now, and each document must be up to 2MB large. Any
       given .JSONL file must be 100MB or smaller, and no more than 20
       files may be given. Sample in-line JSON Lines file (presented
       here with artificial line breaks, but the only actual line break
       is denoted by \\n): { "id": "my_first_id", "text_snippet": {
       "content": "dog car cat"}, "text_features": [ { "text_segment":
       {"start_offset": 4, "end_offset": 6}, "structural_type":
       PARAGRAPH, "bounding_poly": { "normalized_vertices": [ {"x": 0.1,
       "y": 0.1}, {"x": 0.1, "y": 0.3}, {"x": 0.3, "y": 0.3}, {"x": 0.3,
       "y": 0.1}, ] }, } ], }\n { "id": "2", "text_snippet": {
       "content": "An elaborate content", "mime_type": "text/plain" } }
       Sample document JSON Lines file (presented here with artificial
       line breaks, but the only actual line break is denoted by \\n).:
       { "document": { "input_config": { "gcs_source": { "input_uris": [
       "gs://folder/document1.pdf" ] } } } }\n { "document": {
       "input_config": { "gcs_source": { "input_uris": [
       "gs://folder/document2.pdf" ] } } } }

    -  For Tables: Either
       [gcs_source][google.cloud.automl.v1beta1.InputConfig.gcs_source]
       or

    [bigquery_source][google.cloud.automl.v1beta1.InputConfig.bigquery_source].
    GCS case: CSV file(s), each by itself 10GB or smaller and total size
    must be 100GB or smaller, where first file must have a header
    containing column names. If the first row of a subsequent file is
    the same as the header, then it is also treated as a header. All
    other rows contain values for the corresponding columns. The column
    names must contain the model's

    [input_feature_column_specs'][google.cloud.automl.v1beta1.TablesModelMetadata.input_feature_column_specs]

    [display_name-s][google.cloud.automl.v1beta1.ColumnSpec.display_name]
    (order doesn't matter). The columns corresponding to the model's
    input feature column specs must contain values compatible with the
    column spec's data types. Prediction on all the rows, i.e. the CSV
    lines, will be attempted. For FORECASTING

    [prediction_type][google.cloud.automl.v1beta1.TablesModelMetadata.prediction_type]:
    all columns having

    [TIME_SERIES_AVAILABLE_PAST_ONLY][google.cloud.automl.v1beta1.ColumnSpec.ForecastingMetadata.ColumnType]
    type will be ignored. First three sample rows of a CSV file: "First
    Name","Last Name","Dob","Addresses"

    "John","Doe","1968-01-22","[{"status":"current","address":"123_First_Avenue","city":"Seattle","state":"WA","zip":"11111","numberOfYears":"1"},{"status":"previous","address":"456_Main_Street","city":"Portland","state":"OR","zip":"22222","numberOfYears":"5"}]"

    "Jane","Doe","1980-10-16","[{"status":"current","address":"789_Any_Avenue","city":"Albany","state":"NY","zip":"33333","numberOfYears":"2"},{"status":"previous","address":"321_Main_Street","city":"Hoboken","state":"NJ","zip":"44444","numberOfYears":"3"}]}
    BigQuery case: An URI of a BigQuery table. The user data size of the
    BigQuery table must be 100GB or smaller. The column names must
    contain the model's

    [input_feature_column_specs'][google.cloud.automl.v1beta1.TablesModelMetadata.input_feature_column_specs]

    [display_name-s][google.cloud.automl.v1beta1.ColumnSpec.display_name]
    (order doesn't matter). The columns corresponding to the model's
    input feature column specs must contain values compatible with the
    column spec's data types. Prediction on all the rows of the table
    will be attempted. For FORECASTING

    [prediction_type][google.cloud.automl.v1beta1.TablesModelMetadata.prediction_type]:
    all columns having

    [TIME_SERIES_AVAILABLE_PAST_ONLY][google.cloud.automl.v1beta1.ColumnSpec.ForecastingMetadata.ColumnType]
    type will be ignored.

    Definitions: GCS_FILE_PATH = A path to file on GCS, e.g.
    "gs://folder/video.avi". TEXT_SNIPPET = A content of a text snippet,
    UTF-8 encoded, enclosed within double quotes ("") TIME_SEGMENT_START
    = TIME_OFFSET Expresses a beginning, inclusive, of a time segment
    within an example that has a time dimension (e.g. video).
    TIME_SEGMENT_END = TIME_OFFSET Expresses an end, exclusive, of a
    time segment within an example that has a time dimension (e.g.
    video). TIME_OFFSET = A number of seconds as measured from the start
    of an example (e.g. video). Fractions are allowed, up to a
    microsecond precision. "inf" is allowed and it means the end of the
    example.

    Errors: If any of the provided CSV files can't be parsed or if more
    than certain percent of CSV rows cannot be processed then the
    operation fails and prediction does not happen. Regardless of
    overall success or failure the per-row failures, up to a certain
    count cap, will be listed in Operation.metadata.partial_failures.

    This message has `oneof`_ fields (mutually exclusive fields).
    For each oneof, at most one member field can be set at the same time.
    Setting any member of the oneof automatically clears all other
    members.

    .. _oneof: https://proto-plus-python.readthedocs.io/en/stable/fields.html#oneofs-mutually-exclusive-fields

    Attributes:
        gcs_source (google.cloud.automl_v1beta1.types.GcsSource):
            The Google Cloud Storage location for the
            input content.

            This field is a member of `oneof`_ ``source``.
        bigquery_source (google.cloud.automl_v1beta1.types.BigQuerySource):
            The BigQuery location for the input content.

            This field is a member of `oneof`_ ``source``.
    """

    gcs_source: "GcsSource" = proto.Field(
        proto.MESSAGE,
        number=1,
        oneof="source",
        message="GcsSource",
    )
    bigquery_source: "BigQuerySource" = proto.Field(
        proto.MESSAGE,
        number=2,
        oneof="source",
        message="BigQuerySource",
    )


class DocumentInputConfig(proto.Message):
    r"""Input configuration of a
    [Document][google.cloud.automl.v1beta1.Document].

    Attributes:
        gcs_source (google.cloud.automl_v1beta1.types.GcsSource):
            The Google Cloud Storage location of the
            document file. Only a single path should be
            given. Max supported size: 512MB.
            Supported extensions: .PDF.
    """

    gcs_source: "GcsSource" = proto.Field(
        proto.MESSAGE,
        number=1,
        message="GcsSource",
    )


class OutputConfig(proto.Message):
    r"""-  For Translation: CSV file ``translation.csv``, with each line in
       format: ML_USE,GCS_FILE_PATH GCS_FILE_PATH leads to a .TSV file
       which describes examples that have given ML_USE, using the
       following row format per line: TEXT_SNIPPET (in source language)
       \\t TEXT_SNIPPET (in target language)

       -  For Tables: Output depends on whether the dataset was imported
          from GCS or BigQuery. GCS case:

    [gcs_destination][google.cloud.automl.v1beta1.OutputConfig.gcs_destination]
    must be set. Exported are CSV file(s) ``tables_1.csv``,
    ``tables_2.csv``,...,\ ``tables_N.csv`` with each having as header
    line the table's column names, and all other lines contain values
    for the header columns. BigQuery case:

    [bigquery_destination][google.cloud.automl.v1beta1.OutputConfig.bigquery_destination]
    pointing to a BigQuery project must be set. In the given project a
    new dataset will be created with name

    ``export_data_<automl-dataset-display-name>_<timestamp-of-export-call>``
    where will be made BigQuery-dataset-name compatible (e.g. most
    special characters will become underscores), and timestamp will be
    in YYYY_MM_DDThh_mm_ss_sssZ "based on ISO-8601" format. In that
    dataset a new table called ``primary_table`` will be created, and
    filled with precisely the same data as this obtained on import.

    This message has `oneof`_ fields (mutually exclusive fields).
    For each oneof, at most one member field can be set at the same time.
    Setting any member of the oneof automatically clears all other
    members.

    .. _oneof: https://proto-plus-python.readthedocs.io/en/stable/fields.html#oneofs-mutually-exclusive-fields

    Attributes:
        gcs_destination (google.cloud.automl_v1beta1.types.GcsDestination):
            The Google Cloud Storage location where the output is to be
            written to. For Image Object Detection, Text Extraction,
            Video Classification and Tables, in the given directory a
            new directory will be created with name: export_data-- where
            timestamp is in YYYY-MM-DDThh:mm:ss.sssZ ISO-8601 format.
            All export output will be written into that directory.

            This field is a member of `oneof`_ ``destination``.
        bigquery_destination (google.cloud.automl_v1beta1.types.BigQueryDestination):
            The BigQuery location where the output is to
            be written to.

            This field is a member of `oneof`_ ``destination``.
    """

    gcs_destination: "GcsDestination" = proto.Field(
        proto.MESSAGE,
        number=1,
        oneof="destination",
        message="GcsDestination",
    )
    bigquery_destination: "BigQueryDestination" = proto.Field(
        proto.MESSAGE,
        number=2,
        oneof="destination",
        message="BigQueryDestination",
    )


class BatchPredictOutputConfig(proto.Message):
    r"""Output configuration for BatchPredict Action.

    As destination the

    [gcs_destination][google.cloud.automl.v1beta1.BatchPredictOutputConfig.gcs_destination]
    must be set unless specified otherwise for a domain. If
    gcs_destination is set then in the given directory a new directory
    is created. Its name will be "prediction--", where timestamp is in
    YYYY-MM-DDThh:mm:ss.sssZ ISO-8601 format. The contents of it depends
    on the ML problem the predictions are made for.

    -  For Image Classification: In the created directory files
       ``image_classification_1.jsonl``,
       ``image_classification_2.jsonl``,...,\ ``image_classification_N.jsonl``
       will be created, where N may be 1, and depends on the total
       number of the successfully predicted images and annotations. A
       single image will be listed only once with all its annotations,
       and its annotations will never be split across files. Each .JSONL
       file will contain, per line, a JSON representation of a proto
       that wraps image's "ID" : "<id_value>" followed by a list of zero
       or more AnnotationPayload protos (called annotations), which have
       classification detail populated. If prediction for any image
       failed (partially or completely), then an additional
       ``errors_1.jsonl``, ``errors_2.jsonl``,..., ``errors_N.jsonl``
       files will be created (N depends on total number of failed
       predictions). These files will have a JSON representation of a
       proto that wraps the same "ID" : "<id_value>" but here followed
       by exactly one

    [``google.rpc.Status``](https:
    //github.com/googleapis/googleapis/blob/master/google/rpc/status.proto)
    containing only ``code`` and ``message``\ fields.

    -  For Image Object Detection: In the created directory files
       ``image_object_detection_1.jsonl``,
       ``image_object_detection_2.jsonl``,...,\ ``image_object_detection_N.jsonl``
       will be created, where N may be 1, and depends on the total
       number of the successfully predicted images and annotations. Each
       .JSONL file will contain, per line, a JSON representation of a
       proto that wraps image's "ID" : "<id_value>" followed by a list
       of zero or more AnnotationPayload protos (called annotations),
       which have image_object_detection detail populated. A single
       image will be listed only once with all its annotations, and its
       annotations will never be split across files. If prediction for
       any image failed (partially or completely), then additional
       ``errors_1.jsonl``, ``errors_2.jsonl``,..., ``errors_N.jsonl``
       files will be created (N depends on total number of failed
       predictions). These files will have a JSON representation of a
       proto that wraps the same "ID" : "<id_value>" but here followed
       by exactly one

    [``google.rpc.Status``](https:
    //github.com/googleapis/googleapis/blob/master/google/rpc/status.proto)
    containing only ``code`` and ``message``\ fields.

    -  For Video Classification: In the created directory a
       video_classification.csv file, and a .JSON file per each video
       classification requested in the input (i.e. each line in given
       CSV(s)), will be created.

       ::

          The format of video_classification.csv is:

    GCS_FILE_PATH,TIME_SEGMENT_START,TIME_SEGMENT_END,JSON_FILE_NAME,STATUS
    where: GCS_FILE_PATH,TIME_SEGMENT_START,TIME_SEGMENT_END = matches 1
    to 1 the prediction input lines (i.e. video_classification.csv has
    precisely the same number of lines as the prediction input had.)
    JSON_FILE_NAME = Name of .JSON file in the output directory, which
    contains prediction responses for the video time segment. STATUS =
    "OK" if prediction completed successfully, or an error code with
    message otherwise. If STATUS is not "OK" then the .JSON file for
    that line may not exist or be empty.

    ::

            Each .JSON file, assuming STATUS is "OK", will contain a list of
            AnnotationPayload protos in JSON format, which are the predictions
            for the video time segment the file is assigned to in the
            video_classification.csv. All AnnotationPayload protos will have
            video_classification field set, and will be sorted by
            video_classification.type field (note that the returned types are
            governed by `classifaction_types` parameter in
            [PredictService.BatchPredictRequest.params][]).

    -  For Video Object Tracking: In the created directory a
       video_object_tracking.csv file will be created, and multiple
       files video_object_trackinng_1.json,
       video_object_trackinng_2.json,..., video_object_trackinng_N.json,
       where N is the number of requests in the input (i.e. the number
       of lines in given CSV(s)).

       ::

          The format of video_object_tracking.csv is:

    GCS_FILE_PATH,TIME_SEGMENT_START,TIME_SEGMENT_END,JSON_FILE_NAME,STATUS
    where: GCS_FILE_PATH,TIME_SEGMENT_START,TIME_SEGMENT_END = matches 1
    to 1 the prediction input lines (i.e. video_object_tracking.csv has
    precisely the same number of lines as the prediction input had.)
    JSON_FILE_NAME = Name of .JSON file in the output directory, which
    contains prediction responses for the video time segment. STATUS =
    "OK" if prediction completed successfully, or an error code with
    message otherwise. If STATUS is not "OK" then the .JSON file for
    that line may not exist or be empty.

    ::

            Each .JSON file, assuming STATUS is "OK", will contain a list of
            AnnotationPayload protos in JSON format, which are the predictions
            for each frame of the video time segment the file is assigned to in
            video_object_tracking.csv. All AnnotationPayload protos will have
            video_object_tracking field set.

    -  For Text Classification: In the created directory files
       ``text_classification_1.jsonl``,
       ``text_classification_2.jsonl``,...,\ ``text_classification_N.jsonl``
       will be created, where N may be 1, and depends on the total
       number of inputs and annotations found.

       ::

          Each .JSONL file will contain, per line, a JSON representation of a
          proto that wraps input text snippet or input text file and a list of
          zero or more AnnotationPayload protos (called annotations), which
          have classification detail populated. A single text snippet or file
          will be listed only once with all its annotations, and its
          annotations will never be split across files.

          If prediction for any text snippet or file failed (partially or
          completely), then additional `errors_1.jsonl`, `errors_2.jsonl`,...,
          `errors_N.jsonl` files will be created (N depends on total number of
          failed predictions). These files will have a JSON representation of a
          proto that wraps input text snippet or input text file followed by
          exactly one

    [``google.rpc.Status``](https:
    //github.com/googleapis/googleapis/blob/master/google/rpc/status.proto)
    containing only ``code`` and ``message``.

    -  For Text Sentiment: In the created directory files
       ``text_sentiment_1.jsonl``,
       ``text_sentiment_2.jsonl``,...,\ ``text_sentiment_N.jsonl`` will
       be created, where N may be 1, and depends on the total number of
       inputs and annotations found.

       ::

          Each .JSONL file will contain, per line, a JSON representation of a
          proto that wraps input text snippet or input text file and a list of
          zero or more AnnotationPayload protos (called annotations), which
          have text_sentiment detail populated. A single text snippet or file
          will be listed only once with all its annotations, and its
          annotations will never be split across files.

          If prediction for any text snippet or file failed (partially or
          completely), then additional `errors_1.jsonl`, `errors_2.jsonl`,...,
          `errors_N.jsonl` files will be created (N depends on total number of
          failed predictions). These files will have a JSON representation of a
          proto that wraps input text snippet or input text file followed by
          exactly one

    [``google.rpc.Status``](https:
    //github.com/googleapis/googleapis/blob/master/google/rpc/status.proto)
    containing only ``code`` and ``message``.

    -  For Text Extraction: In the created directory files
       ``text_extraction_1.jsonl``,
       ``text_extraction_2.jsonl``,...,\ ``text_extraction_N.jsonl``
       will be created, where N may be 1, and depends on the total
       number of inputs and annotations found. The contents of these
       .JSONL file(s) depend on whether the input used inline text, or
       documents. If input was inline, then each .JSONL file will
       contain, per line, a JSON representation of a proto that wraps
       given in request text snippet's "id" (if specified), followed by
       input text snippet, and a list of zero or more AnnotationPayload
       protos (called annotations), which have text_extraction detail
       populated. A single text snippet will be listed only once with
       all its annotations, and its annotations will never be split
       across files. If input used documents, then each .JSONL file will
       contain, per line, a JSON representation of a proto that wraps
       given in request document proto, followed by its OCR-ed
       representation in the form of a text snippet, finally followed by
       a list of zero or more AnnotationPayload protos (called
       annotations), which have text_extraction detail populated and
       refer, via their indices, to the OCR-ed text snippet. A single
       document (and its text snippet) will be listed only once with all
       its annotations, and its annotations will never be split across
       files. If prediction for any text snippet failed (partially or
       completely), then additional ``errors_1.jsonl``,
       ``errors_2.jsonl``,..., ``errors_N.jsonl`` files will be created
       (N depends on total number of failed predictions). These files
       will have a JSON representation of a proto that wraps either the
       "id" : "<id_value>" (in case of inline) or the document proto (in
       case of document) but here followed by exactly one

    [``google.rpc.Status``](https:
    //github.com/googleapis/googleapis/blob/master/google/rpc/status.proto)
    containing only ``code`` and ``message``.

    -  For Tables: Output depends on whether

    [gcs_destination][google.cloud.automl.v1beta1.BatchPredictOutputConfig.gcs_destination]
    or

    [bigquery_destination][google.cloud.automl.v1beta1.BatchPredictOutputConfig.bigquery_destination]
    is set (either is allowed). GCS case: In the created directory files
    ``tables_1.csv``, ``tables_2.csv``,..., ``tables_N.csv`` will be
    created, where N may be 1, and depends on the total number of the
    successfully predicted rows. For all CLASSIFICATION

    [prediction_type-s][google.cloud.automl.v1beta1.TablesModelMetadata.prediction_type]:
    Each .csv file will contain a header, listing all columns'

    [display_name-s][google.cloud.automl.v1beta1.ColumnSpec.display_name]
    given on input followed by M target column names in the format of

    "<[target_column_specs][google.cloud.automl.v1beta1.TablesModelMetadata.target_column_spec]

    [display_name][google.cloud.automl.v1beta1.ColumnSpec.display_name]>\_\_score"
    where M is the number of distinct target values, i.e. number of
    distinct values in the target column of the table used to train the
    model. Subsequent lines will contain the respective values of
    successfully predicted rows, with the last, i.e. the target, columns
    having the corresponding prediction
    [scores][google.cloud.automl.v1beta1.TablesAnnotation.score]. For
    REGRESSION and FORECASTING

    [prediction_type-s][google.cloud.automl.v1beta1.TablesModelMetadata.prediction_type]:
    Each .csv file will contain a header, listing all columns'
    [display_name-s][google.cloud.automl.v1beta1.display_name] given on
    input followed by the predicted target column with name in the
    format of

    "predicted_<[target_column_specs][google.cloud.automl.v1beta1.TablesModelMetadata.target_column_spec]

    [display_name][google.cloud.automl.v1beta1.ColumnSpec.display_name]>"
    Subsequent lines will contain the respective values of successfully
    predicted rows, with the last, i.e. the target, column having the
    predicted target value. If prediction for any rows failed, then an
    additional ``errors_1.csv``, ``errors_2.csv``,..., ``errors_N.csv``
    will be created (N depends on total number of failed rows). These
    files will have analogous format as ``tables_*.csv``, but always
    with a single target column having

    [``google.rpc.Status``](https:
    //github.com/googleapis/googleapis/blob/master/google/rpc/status.proto)
    represented as a JSON string, and containing only ``code`` and
    ``message``. BigQuery case:

    [bigquery_destination][google.cloud.automl.v1beta1.OutputConfig.bigquery_destination]
    pointing to a BigQuery project must be set. In the given project a
    new dataset will be created with name
    ``prediction_<model-display-name>_<timestamp-of-prediction-call>``
    where will be made BigQuery-dataset-name compatible (e.g. most
    special characters will become underscores), and timestamp will be
    in YYYY_MM_DDThh_mm_ss_sssZ "based on ISO-8601" format. In the
    dataset two tables will be created, ``predictions``, and ``errors``.
    The ``predictions`` table's column names will be the input columns'

    [display_name-s][google.cloud.automl.v1beta1.ColumnSpec.display_name]
    followed by the target column with name in the format of

    "predicted_<[target_column_specs][google.cloud.automl.v1beta1.TablesModelMetadata.target_column_spec]

    [display_name][google.cloud.automl.v1beta1.ColumnSpec.display_name]>"
    The input feature columns will contain the respective values of
    successfully predicted rows, with the target column having an ARRAY
    of

    [AnnotationPayloads][google.cloud.automl.v1beta1.AnnotationPayload],
    represented as STRUCT-s, containing
    [TablesAnnotation][google.cloud.automl.v1beta1.TablesAnnotation].
    The ``errors`` table contains rows for which the prediction has
    failed, it has analogous input columns while the target column name
    is in the format of

    "errors_<[target_column_specs][google.cloud.automl.v1beta1.TablesModelMetadata.target_column_spec]

    [display_name][google.cloud.automl.v1beta1.ColumnSpec.display_name]>",
    and as a value has

    [``google.rpc.Status``](https:
    //github.com/googleapis/googleapis/blob/master/google/rpc/status.proto)
    represented as a STRUCT, and containing only ``code`` and
    ``message``.

    This message has `oneof`_ fields (mutually exclusive fields).
    For each oneof, at most one member field can be set at the same time.
    Setting any member of the oneof automatically clears all other
    members.

    .. _oneof: https://proto-plus-python.readthedocs.io/en/stable/fields.html#oneofs-mutually-exclusive-fields

    Attributes:
        gcs_destination (google.cloud.automl_v1beta1.types.GcsDestination):
            The Google Cloud Storage location of the
            directory where the output is to be written to.

            This field is a member of `oneof`_ ``destination``.
        bigquery_destination (google.cloud.automl_v1beta1.types.BigQueryDestination):
            The BigQuery location where the output is to
            be written to.

            This field is a member of `oneof`_ ``destination``.
    """

    gcs_destination: "GcsDestination" = proto.Field(
        proto.MESSAGE,
        number=1,
        oneof="destination",
        message="GcsDestination",
    )
    bigquery_destination: "BigQueryDestination" = proto.Field(
        proto.MESSAGE,
        number=2,
        oneof="destination",
        message="BigQueryDestination",
    )


class ModelExportOutputConfig(proto.Message):
    r"""Output configuration for ModelExport Action.

    This message has `oneof`_ fields (mutually exclusive fields).
    For each oneof, at most one member field can be set at the same time.
    Setting any member of the oneof automatically clears all other
    members.

    .. _oneof: https://proto-plus-python.readthedocs.io/en/stable/fields.html#oneofs-mutually-exclusive-fields

    Attributes:
        gcs_destination (google.cloud.automl_v1beta1.types.GcsDestination):
            The Google Cloud Storage location where the model is to be
            written to. This location may only be set for the following
            model formats: "tflite", "edgetpu_tflite", "tf_saved_model",
            "tf_js", "core_ml".

            Under the directory given as the destination a new one with
            name "model-export--", where timestamp is in
            YYYY-MM-DDThh:mm:ss.sssZ ISO-8601 format, will be created.
            Inside the model and any of its supporting files will be
            written.

            This field is a member of `oneof`_ ``destination``.
        gcr_destination (google.cloud.automl_v1beta1.types.GcrDestination):
            The GCR location where model image is to be
            pushed to. This location may only be set for the
            following model formats:

              "docker".

            The model image will be created under the given
            URI.

            This field is a member of `oneof`_ ``destination``.
        model_format (str):
            The format in which the model must be exported. The
            available, and default, formats depend on the problem and
            model type (if given problem and type combination doesn't
            have a format listed, it means its models are not
            exportable):

            -  For Image Classification mobile-low-latency-1,
               mobile-versatile-1, mobile-high-accuracy-1: "tflite"
               (default), "edgetpu_tflite", "tf_saved_model", "tf_js",
               "docker".

            -  For Image Classification mobile-core-ml-low-latency-1,
               mobile-core-ml-versatile-1,
               mobile-core-ml-high-accuracy-1: "core_ml" (default).

            -  For Image Object Detection mobile-low-latency-1,
               mobile-versatile-1, mobile-high-accuracy-1: "tflite",
               "tf_saved_model", "tf_js".

            -  For Video Classification cloud, "tf_saved_model".

            -  For Video Object Tracking cloud, "tf_saved_model".

            -  For Video Object Tracking mobile-versatile-1: "tflite",
               "edgetpu_tflite", "tf_saved_model", "docker".

            -  For Video Object Tracking mobile-coral-versatile-1:
               "tflite", "edgetpu_tflite", "docker".

            -  For Video Object Tracking mobile-coral-low-latency-1:
               "tflite", "edgetpu_tflite", "docker".

            -  For Video Object Tracking mobile-jetson-versatile-1:
               "tf_saved_model", "docker".

            -  For Tables: "docker".

            Formats description:

            -  tflite - Used for Android mobile devices.
            -  edgetpu_tflite - Used for `Edge
               TPU <https://cloud.google.com/edge-tpu/>`__ devices.
            -  tf_saved_model - A tensorflow model in SavedModel format.
            -  tf_js - A
               `TensorFlow.js <https://www.tensorflow.org/js>`__ model
               that can be used in the browser and in Node.js using
               JavaScript.
            -  docker - Used for Docker containers. Use the params field
               to customize the container. The container is verified to
               work correctly on ubuntu 16.04 operating system. See more
               at [containers

            quickstart](https:
            //cloud.google.com/vision/automl/docs/containers-gcs-quickstart)

            -  core_ml - Used for iOS mobile devices.
        params (MutableMapping[str, str]):
            Additional model-type and format specific parameters
            describing the requirements for the to be exported model
            files, any string must be up to 25000 characters long.

            -  For ``docker`` format: ``cpu_architecture`` - (string)
               "x86_64" (default). ``gpu_architecture`` - (string)
               "none" (default), "nvidia".
    """

    gcs_destination: "GcsDestination" = proto.Field(
        proto.MESSAGE,
        number=1,
        oneof="destination",
        message="GcsDestination",
    )
    gcr_destination: "GcrDestination" = proto.Field(
        proto.MESSAGE,
        number=3,
        oneof="destination",
        message="GcrDestination",
    )
    model_format: str = proto.Field(
        proto.STRING,
        number=4,
    )
    params: MutableMapping[str, str] = proto.MapField(
        proto.STRING,
        proto.STRING,
        number=2,
    )


class ExportEvaluatedExamplesOutputConfig(proto.Message):
    r"""Output configuration for ExportEvaluatedExamples Action. Note that
    this call is available only for 30 days since the moment the model
    was evaluated. The output depends on the domain, as follows (note
    that only examples from the TEST set are exported):

    -  For Tables:

    [bigquery_destination][google.cloud.automl.v1beta1.OutputConfig.bigquery_destination]
    pointing to a BigQuery project must be set. In the given project a
    new dataset will be created with name

    ``export_evaluated_examples_<model-display-name>_<timestamp-of-export-call>``
    where will be made BigQuery-dataset-name compatible (e.g. most
    special characters will become underscores), and timestamp will be
    in YYYY_MM_DDThh_mm_ss_sssZ "based on ISO-8601" format. In the
    dataset an ``evaluated_examples`` table will be created. It will
    have all the same columns as the

    [primary_table][google.cloud.automl.v1beta1.TablesDatasetMetadata.primary_table_spec_id]
    of the [dataset][google.cloud.automl.v1beta1.Model.dataset_id] from
    which the model was created, as they were at the moment of model's
    evaluation (this includes the target column with its ground truth),
    followed by a column called "predicted_<target_column>". That last
    column will contain the model's prediction result for each
    respective row, given as ARRAY of
    [AnnotationPayloads][google.cloud.automl.v1beta1.AnnotationPayload],
    represented as STRUCT-s, containing
    [TablesAnnotation][google.cloud.automl.v1beta1.TablesAnnotation].


    .. _oneof: https://proto-plus-python.readthedocs.io/en/stable/fields.html#oneofs-mutually-exclusive-fields

    Attributes:
        bigquery_destination (google.cloud.automl_v1beta1.types.BigQueryDestination):
            The BigQuery location where the output is to
            be written to.

            This field is a member of `oneof`_ ``destination``.
    """

    bigquery_destination: "BigQueryDestination" = proto.Field(
        proto.MESSAGE,
        number=2,
        oneof="destination",
        message="BigQueryDestination",
    )


class GcsSource(proto.Message):
    r"""The Google Cloud Storage location for the input content.

    Attributes:
        input_uris (MutableSequence[str]):
            Required. Google Cloud Storage URIs to input files, up to
            2000 characters long. Accepted forms:

            -  Full object path, e.g. gs://bucket/directory/object.csv
    """

    input_uris: MutableSequence[str] = proto.RepeatedField(
        proto.STRING,
        number=1,
    )


class BigQuerySource(proto.Message):
    r"""The BigQuery location for the input content.

    Attributes:
        input_uri (str):
            Required. BigQuery URI to a table, up to 2000 characters
            long. Accepted forms:

            -  BigQuery path e.g. bq://projectId.bqDatasetId.bqTableId
    """

    input_uri: str = proto.Field(
        proto.STRING,
        number=1,
    )


class GcsDestination(proto.Message):
    r"""The Google Cloud Storage location where the output is to be
    written to.

    Attributes:
        output_uri_prefix (str):
            Required. Google Cloud Storage URI to output directory, up
            to 2000 characters long. Accepted forms:

            -  Prefix path: gs://bucket/directory The requesting user
               must have write permission to the bucket. The directory
               is created if it doesn't exist.
    """

    output_uri_prefix: str = proto.Field(
        proto.STRING,
        number=1,
    )


class BigQueryDestination(proto.Message):
    r"""The BigQuery location for the output content.

    Attributes:
        output_uri (str):
            Required. BigQuery URI to a project, up to 2000 characters
            long. Accepted forms:

            -  BigQuery path e.g. bq://projectId
    """

    output_uri: str = proto.Field(
        proto.STRING,
        number=1,
    )


class GcrDestination(proto.Message):
    r"""The GCR location where the image must be pushed to.

    Attributes:
        output_uri (str):
            Required. Google Contained Registry URI of the new image, up
            to 2000 characters long. See

            https: //cloud.google.com/container-registry/do //
            cs/pushing-and-pulling#pushing_an_image_to_a_registry
            Accepted forms:

            -  [HOSTNAME]/[PROJECT-ID]/[IMAGE]
            -  [HOSTNAME]/[PROJECT-ID]/[IMAGE]:[TAG]

            The requesting user must have permission to push images the
            project.
    """

    output_uri: str = proto.Field(
        proto.STRING,
        number=1,
    )


__all__ = tuple(sorted(__protobuf__.manifest))



# -*- coding: utf-8 -*-
# Copyright 2024 Google LLC
#
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
#     http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.
#
import json  # type: ignore
import re
from typing import Any, Callable, Dict, List, Optional, Sequence, Tuple, Union

from google.api_core import gapic_v1, path_template
from google.longrunning import operations_pb2  # type: ignore
from google.protobuf import json_format

from google.cloud.automl_v1beta1.types import annotation_spec
from google.cloud.automl_v1beta1.types import column_spec
from google.cloud.automl_v1beta1.types import column_spec as gca_column_spec
from google.cloud.automl_v1beta1.types import dataset
from google.cloud.automl_v1beta1.types import dataset as gca_dataset
from google.cloud.automl_v1beta1.types import model, model_evaluation, service
from google.cloud.automl_v1beta1.types import table_spec
from google.cloud.automl_v1beta1.types import table_spec as gca_table_spec

from .base import DEFAULT_CLIENT_INFO, AutoMlTransport


class _BaseAutoMlRestTransport(AutoMlTransport):
    """Base REST backend transport for AutoMl.

    Note: This class is not meant to be used directly. Use its sync and
    async sub-classes instead.

    This class defines the same methods as the primary client, so the
    primary client can load the underlying transport implementation
    and call it.

    It sends JSON representations of protocol buffers over HTTP/1.1
    """

    def __init__(
        self,
        *,
        host: str = "automl.googleapis.com",
        credentials: Optional[Any] = None,
        client_info: gapic_v1.client_info.ClientInfo = DEFAULT_CLIENT_INFO,
        always_use_jwt_access: Optional[bool] = False,
        url_scheme: str = "https",
        api_audience: Optional[str] = None,
    ) -> None:
        """Instantiate the transport.
        Args:
            host (Optional[str]):
                 The hostname to connect to (default: 'automl.googleapis.com').
            credentials (Optional[Any]): The
                authorization credentials to attach to requests. These
                credentials identify the application to the service; if none
                are specified, the client will attempt to ascertain the
                credentials from the environment.
            client_info (google.api_core.gapic_v1.client_info.ClientInfo):
                The client info used to send a user-agent string along with
                API requests. If ``None``, then default info will be used.
                Generally, you only need to set this if you are developing
                your own client library.
            always_use_jwt_access (Optional[bool]): Whether self signed JWT should
                be used for service account credentials.
            url_scheme: the protocol scheme for the API endpoint.  Normally
                "https", but for testing or local servers,
                "http" can be specified.
        """
        # Run the base constructor
        maybe_url_match = re.match("^(?P<scheme>http(?:s)?://)?(?P<host>.*)$", host)
        if maybe_url_match is None:
            raise ValueError(
                f"Unexpected hostname structure: {host}"
            )  # pragma: NO COVER

        url_match_items = maybe_url_match.groupdict()

        host = f"{url_scheme}://{host}" if not url_match_items["scheme"] else host

        super().__init__(
            host=host,
            credentials=credentials,
            client_info=client_info,
            always_use_jwt_access=always_use_jwt_access,
            api_audience=api_audience,
        )

    class _BaseCreateDataset:
        def __hash__(self):  # pragma: NO COVER
            return NotImplementedError("__hash__ must be implemented.")

        __REQUIRED_FIELDS_DEFAULT_VALUES: Dict[str, Any] = {}

        @classmethod
        def _get_unset_required_fields(cls, message_dict):
            return {
                k: v
                for k, v in cls.__REQUIRED_FIELDS_DEFAULT_VALUES.items()
                if k not in message_dict
            }

        @staticmethod
        def _get_http_options():
            http_options: List[Dict[str, str]] = [
                {
                    "method": "post",
                    "uri": "/v1beta1/{parent=projects/*/locations/*}/datasets",
                    "body": "dataset",
                },
            ]
            return http_options

        @staticmethod
        def _get_transcoded_request(http_options, request):
            pb_request = service.CreateDatasetRequest.pb(request)
            transcoded_request = path_template.transcode(http_options, pb_request)
            return transcoded_request

        @staticmethod
        def _get_request_body_json(transcoded_request):
            # Jsonify the request body

            body = json_format.MessageToJson(
                transcoded_request["body"], use_integers_for_enums=True
            )
            return body

        @staticmethod
        def _get_query_params_json(transcoded_request):
            query_params = json.loads(
                json_format.MessageToJson(
                    transcoded_request["query_params"],
                    use_integers_for_enums=True,
                )
            )
            query_params.update(
                _BaseAutoMlRestTransport._BaseCreateDataset._get_unset_required_fields(
                    query_params
                )
            )

            query_params["$alt"] = "json;enum-encoding=int"
            return query_params

    class _BaseCreateModel:
        def __hash__(self):  # pragma: NO COVER
            return NotImplementedError("__hash__ must be implemented.")

        __REQUIRED_FIELDS_DEFAULT_VALUES: Dict[str, Any] = {}

        @classmethod
        def _get_unset_required_fields(cls, message_dict):
            return {
                k: v
                for k, v in cls.__REQUIRED_FIELDS_DEFAULT_VALUES.items()
                if k not in message_dict
            }

        @staticmethod
        def _get_http_options():
            http_options: List[Dict[str, str]] = [
                {
                    "method": "post",
                    "uri": "/v1beta1/{parent=projects/*/locations/*}/models",
                    "body": "model",
                },
            ]
            return http_options

        @staticmethod
        def _get_transcoded_request(http_options, request):
            pb_request = service.CreateModelRequest.pb(request)
            transcoded_request = path_template.transcode(http_options, pb_request)
            return transcoded_request

        @staticmethod
        def _get_request_body_json(transcoded_request):
            # Jsonify the request body

            body = json_format.MessageToJson(
                transcoded_request["body"], use_integers_for_enums=True
            )
            return body

        @staticmethod
        def _get_query_params_json(transcoded_request):
            query_params = json.loads(
                json_format.MessageToJson(
                    transcoded_request["query_params"],
                    use_integers_for_enums=True,
                )
            )
            query_params.update(
                _BaseAutoMlRestTransport._BaseCreateModel._get_unset_required_fields(
                    query_params
                )
            )

            query_params["$alt"] = "json;enum-encoding=int"
            return query_params

    class _BaseDeleteDataset:
        def __hash__(self):  # pragma: NO COVER
            return NotImplementedError("__hash__ must be implemented.")

        __REQUIRED_FIELDS_DEFAULT_VALUES: Dict[str, Any] = {}

        @classmethod
        def _get_unset_required_fields(cls, message_dict):
            return {
                k: v
                for k, v in cls.__REQUIRED_FIELDS_DEFAULT_VALUES.items()
                if k not in message_dict
            }

        @staticmethod
        def _get_http_options():
            http_options: List[Dict[str, str]] = [
                {
                    "method": "delete",
                    "uri": "/v1beta1/{name=projects/*/locations/*/datasets/*}",
                },
            ]
            return http_options

        @staticmethod
        def _get_transcoded_request(http_options, request):
            pb_request = service.DeleteDatasetRequest.pb(request)
            transcoded_request = path_template.transcode(http_options, pb_request)
            return transcoded_request

        @staticmethod
        def _get_query_params_json(transcoded_request):
            query_params = json.loads(
                json_format.MessageToJson(
                    transcoded_request["query_params"],
                    use_integers_for_enums=True,
                )
            )
            query_params.update(
                _BaseAutoMlRestTransport._BaseDeleteDataset._get_unset_required_fields(
                    query_params
                )
            )

            query_params["$alt"] = "json;enum-encoding=int"
            return query_params

    class _BaseDeleteModel:
        def __hash__(self):  # pragma: NO COVER
            return NotImplementedError("__hash__ must be implemented.")

        __REQUIRED_FIELDS_DEFAULT_VALUES: Dict[str, Any] = {}

        @classmethod
        def _get_unset_required_fields(cls, message_dict):
            return {
                k: v
                for k, v in cls.__REQUIRED_FIELDS_DEFAULT_VALUES.items()
                if k not in message_dict
            }

        @staticmethod
        def _get_http_options():
            http_options: List[Dict[str, str]] = [
                {
                    "method": "delete",
                    "uri": "/v1beta1/{name=projects/*/locations/*/models/*}",
                },
            ]
            return http_options

        @staticmethod
        def _get_transcoded_request(http_options, request):
            pb_request = service.DeleteModelRequest.pb(request)
            transcoded_request = path_template.transcode(http_options, pb_request)
            return transcoded_request

        @staticmethod
        def _get_query_params_json(transcoded_request):
            query_params = json.loads(
                json_format.MessageToJson(
                    transcoded_request["query_params"],
                    use_integers_for_enums=True,
                )
            )
            query_params.update(
                _BaseAutoMlRestTransport._BaseDeleteModel._get_unset_required_fields(
                    query_params
                )
            )

            query_params["$alt"] = "json;enum-encoding=int"
            return query_params

    class _BaseDeployModel:
        def __hash__(self):  # pragma: NO COVER
            return NotImplementedError("__hash__ must be implemented.")

        __REQUIRED_FIELDS_DEFAULT_VALUES: Dict[str, Any] = {}

        @classmethod
        def _get_unset_required_fields(cls, message_dict):
            return {
                k: v
                for k, v in cls.__REQUIRED_FIELDS_DEFAULT_VALUES.items()
                if k not in message_dict
            }

        @staticmethod
        def _get_http_options():
            http_options: List[Dict[str, str]] = [
                {
                    "method": "post",
                    "uri": "/v1beta1/{name=projects/*/locations/*/models/*}:deploy",
                    "body": "*",
                },
            ]
            return http_options

        @staticmethod
        def _get_transcoded_request(http_options, request):
            pb_request = service.DeployModelRequest.pb(request)
            transcoded_request = path_template.transcode(http_options, pb_request)
            return transcoded_request

        @staticmethod
        def _get_request_body_json(transcoded_request):
            # Jsonify the request body

            body = json_format.MessageToJson(
                transcoded_request["body"], use_integers_for_enums=True
            )
            return body

        @staticmethod
        def _get_query_params_json(transcoded_request):
            query_params = json.loads(
                json_format.MessageToJson(
                    transcoded_request["query_params"],
                    use_integers_for_enums=True,
                )
            )
            query_params.update(
                _BaseAutoMlRestTransport._BaseDeployModel._get_unset_required_fields(
                    query_params
                )
            )

            query_params["$alt"] = "json;enum-encoding=int"
            return query_params

    class _BaseExportData:
        def __hash__(self):  # pragma: NO COVER
            return NotImplementedError("__hash__ must be implemented.")

        __REQUIRED_FIELDS_DEFAULT_VALUES: Dict[str, Any] = {}

        @classmethod
        def _get_unset_required_fields(cls, message_dict):
            return {
                k: v
                for k, v in cls.__REQUIRED_FIELDS_DEFAULT_VALUES.items()
                if k not in message_dict
            }

        @staticmethod
        def _get_http_options():
            http_options: List[Dict[str, str]] = [
                {
                    "method": "post",
                    "uri": "/v1beta1/{name=projects/*/locations/*/datasets/*}:exportData",
                    "body": "*",
                },
            ]
            return http_options

        @staticmethod
        def _get_transcoded_request(http_options, request):
            pb_request = service.ExportDataRequest.pb(request)
            transcoded_request = path_template.transcode(http_options, pb_request)
            return transcoded_request

        @staticmethod
        def _get_request_body_json(transcoded_request):
            # Jsonify the request body

            body = json_format.MessageToJson(
                transcoded_request["body"], use_integers_for_enums=True
            )
            return body

        @staticmethod
        def _get_query_params_json(transcoded_request):
            query_params = json.loads(
                json_format.MessageToJson(
                    transcoded_request["query_params"],
                    use_integers_for_enums=True,
                )
            )
            query_params.update(
                _BaseAutoMlRestTransport._BaseExportData._get_unset_required_fields(
                    query_params
                )
            )

            query_params["$alt"] = "json;enum-encoding=int"
            return query_params

    class _BaseExportEvaluatedExamples:
        def __hash__(self):  # pragma: NO COVER
            return NotImplementedError("__hash__ must be implemented.")

        __REQUIRED_FIELDS_DEFAULT_VALUES: Dict[str, Any] = {}

        @classmethod
        def _get_unset_required_fields(cls, message_dict):
            return {
                k: v
                for k, v in cls.__REQUIRED_FIELDS_DEFAULT_VALUES.items()
                if k not in message_dict
            }

        @staticmethod
        def _get_http_options():
            http_options: List[Dict[str, str]] = [
                {
                    "method": "post",
                    "uri": "/v1beta1/{name=projects/*/locations/*/models/*}:exportEvaluatedExamples",
                    "body": "*",
                },
            ]
            return http_options

        @staticmethod
        def _get_transcoded_request(http_options, request):
            pb_request = service.ExportEvaluatedExamplesRequest.pb(request)
            transcoded_request = path_template.transcode(http_options, pb_request)
            return transcoded_request

        @staticmethod
        def _get_request_body_json(transcoded_request):
            # Jsonify the request body

            body = json_format.MessageToJson(
                transcoded_request["body"], use_integers_for_enums=True
            )
            return body

        @staticmethod
        def _get_query_params_json(transcoded_request):
            query_params = json.loads(
                json_format.MessageToJson(
                    transcoded_request["query_params"],
                    use_integers_for_enums=True,
                )
            )
            query_params.update(
                _BaseAutoMlRestTransport._BaseExportEvaluatedExamples._get_unset_required_fields(
                    query_params
                )
            )

            query_params["$alt"] = "json;enum-encoding=int"
            return query_params

    class _BaseExportModel:
        def __hash__(self):  # pragma: NO COVER
            return NotImplementedError("__hash__ must be implemented.")

        __REQUIRED_FIELDS_DEFAULT_VALUES: Dict[str, Any] = {}

        @classmethod
        def _get_unset_required_fields(cls, message_dict):
            return {
                k: v
                for k, v in cls.__REQUIRED_FIELDS_DEFAULT_VALUES.items()
                if k not in message_dict
            }

        @staticmethod
        def _get_http_options():
            http_options: List[Dict[str, str]] = [
                {
                    "method": "post",
                    "uri": "/v1beta1/{name=projects/*/locations/*/models/*}:export",
                    "body": "*",
                },
            ]
            return http_options

        @staticmethod
        def _get_transcoded_request(http_options, request):
            pb_request = service.ExportModelRequest.pb(request)
            transcoded_request = path_template.transcode(http_options, pb_request)
            return transcoded_request

        @staticmethod
        def _get_request_body_json(transcoded_request):
            # Jsonify the request body

            body = json_format.MessageToJson(
                transcoded_request["body"], use_integers_for_enums=True
            )
            return body

        @staticmethod
        def _get_query_params_json(transcoded_request):
            query_params = json.loads(
                json_format.MessageToJson(
                    transcoded_request["query_params"],
                    use_integers_for_enums=True,
                )
            )
            query_params.update(
                _BaseAutoMlRestTransport._BaseExportModel._get_unset_required_fields(
                    query_params
                )
            )

            query_params["$alt"] = "json;enum-encoding=int"
            return query_params

    class _BaseGetAnnotationSpec:
        def __hash__(self):  # pragma: NO COVER
            return NotImplementedError("__hash__ must be implemented.")

        __REQUIRED_FIELDS_DEFAULT_VALUES: Dict[str, Any] = {}

        @classmethod
        def _get_unset_required_fields(cls, message_dict):
            return {
                k: v
                for k, v in cls.__REQUIRED_FIELDS_DEFAULT_VALUES.items()
                if k not in message_dict
            }

        @staticmethod
        def _get_http_options():
            http_options: List[Dict[str, str]] = [
                {
                    "method": "get",
                    "uri": "/v1beta1/{name=projects/*/locations/*/datasets/*/annotationSpecs/*}",
                },
            ]
            return http_options

        @staticmethod
        def _get_transcoded_request(http_options, request):
            pb_request = service.GetAnnotationSpecRequest.pb(request)
            transcoded_request = path_template.transcode(http_options, pb_request)
            return transcoded_request

        @staticmethod
        def _get_query_params_json(transcoded_request):
            query_params = json.loads(
                json_format.MessageToJson(
                    transcoded_request["query_params"],
                    use_integers_for_enums=True,
                )
            )
            query_params.update(
                _BaseAutoMlRestTransport._BaseGetAnnotationSpec._get_unset_required_fields(
                    query_params
                )
            )

            query_params["$alt"] = "json;enum-encoding=int"
            return query_params

    class _BaseGetColumnSpec:
        def __hash__(self):  # pragma: NO COVER
            return NotImplementedError("__hash__ must be implemented.")

        __REQUIRED_FIELDS_DEFAULT_VALUES: Dict[str, Any] = {}

        @classmethod
        def _get_unset_required_fields(cls, message_dict):
            return {
                k: v
                for k, v in cls.__REQUIRED_FIELDS_DEFAULT_VALUES.items()
                if k not in message_dict
            }

        @staticmethod
        def _get_http_options():
            http_options: List[Dict[str, str]] = [
                {
                    "method": "get",
                    "uri": "/v1beta1/{name=projects/*/locations/*/datasets/*/tableSpecs/*/columnSpecs/*}",
                },
            ]
            return http_options

        @staticmethod
        def _get_transcoded_request(http_options, request):
            pb_request = service.GetColumnSpecRequest.pb(request)
            transcoded_request = path_template.transcode(http_options, pb_request)
            return transcoded_request

        @staticmethod
        def _get_query_params_json(transcoded_request):
            query_params = json.loads(
                json_format.MessageToJson(
                    transcoded_request["query_params"],
                    use_integers_for_enums=True,
                )
            )
            query_params.update(
                _BaseAutoMlRestTransport._BaseGetColumnSpec._get_unset_required_fields(
                    query_params
                )
            )

            query_params["$alt"] = "json;enum-encoding=int"
            return query_params

    class _BaseGetDataset:
        def __hash__(self):  # pragma: NO COVER
            return NotImplementedError("__hash__ must be implemented.")

        __REQUIRED_FIELDS_DEFAULT_VALUES: Dict[str, Any] = {}

        @classmethod
        def _get_unset_required_fields(cls, message_dict):
            return {
                k: v
                for k, v in cls.__REQUIRED_FIELDS_DEFAULT_VALUES.items()
                if k not in message_dict
            }

        @staticmethod
        def _get_http_options():
            http_options: List[Dict[str, str]] = [
                {
                    "method": "get",
                    "uri": "/v1beta1/{name=projects/*/locations/*/datasets/*}",
                },
            ]
            return http_options

        @staticmethod
        def _get_transcoded_request(http_options, request):
            pb_request = service.GetDatasetRequest.pb(request)
            transcoded_request = path_template.transcode(http_options, pb_request)
            return transcoded_request

        @staticmethod
        def _get_query_params_json(transcoded_request):
            query_params = json.loads(
                json_format.MessageToJson(
                    transcoded_request["query_params"],
                    use_integers_for_enums=True,
                )
            )
            query_params.update(
                _BaseAutoMlRestTransport._BaseGetDataset._get_unset_required_fields(
                    query_params
                )
            )

            query_params["$alt"] = "json;enum-encoding=int"
            return query_params

    class _BaseGetModel:
        def __hash__(self):  # pragma: NO COVER
            return NotImplementedError("__hash__ must be implemented.")

        __REQUIRED_FIELDS_DEFAULT_VALUES: Dict[str, Any] = {}

        @classmethod
        def _get_unset_required_fields(cls, message_dict):
            return {
                k: v
                for k, v in cls.__REQUIRED_FIELDS_DEFAULT_VALUES.items()
                if k not in message_dict
            }

        @staticmethod
        def _get_http_options():
            http_options: List[Dict[str, str]] = [
                {
                    "method": "get",
                    "uri": "/v1beta1/{name=projects/*/locations/*/models/*}",
                },
            ]
            return http_options

        @staticmethod
        def _get_transcoded_request(http_options, request):
            pb_request = service.GetModelRequest.pb(request)
            transcoded_request = path_template.transcode(http_options, pb_request)
            return transcoded_request

        @staticmethod
        def _get_query_params_json(transcoded_request):
            query_params = json.loads(
                json_format.MessageToJson(
                    transcoded_request["query_params"],
                    use_integers_for_enums=True,
                )
            )
            query_params.update(
                _BaseAutoMlRestTransport._BaseGetModel._get_unset_required_fields(
                    query_params
                )
            )

            query_params["$alt"] = "json;enum-encoding=int"
            return query_params

    class _BaseGetModelEvaluation:
        def __hash__(self):  # pragma: NO COVER
            return NotImplementedError("__hash__ must be implemented.")

        __REQUIRED_FIELDS_DEFAULT_VALUES: Dict[str, Any] = {}

        @classmethod
        def _get_unset_required_fields(cls, message_dict):
            return {
                k: v
                for k, v in cls.__REQUIRED_FIELDS_DEFAULT_VALUES.items()
                if k not in message_dict
            }

        @staticmethod
        def _get_http_options():
            http_options: List[Dict[str, str]] = [
                {
                    "method": "get",
                    "uri": "/v1beta1/{name=projects/*/locations/*/models/*/modelEvaluations/*}",
                },
            ]
            return http_options

        @staticmethod
        def _get_transcoded_request(http_options, request):
            pb_request = service.GetModelEvaluationRequest.pb(request)
            transcoded_request = path_template.transcode(http_options, pb_request)
            return transcoded_request

        @staticmethod
        def _get_query_params_json(transcoded_request):
            query_params = json.loads(
                json_format.MessageToJson(
                    transcoded_request["query_params"],
                    use_integers_for_enums=True,
                )
            )
            query_params.update(
                _BaseAutoMlRestTransport._BaseGetModelEvaluation._get_unset_required_fields(
                    query_params
                )
            )

            query_params["$alt"] = "json;enum-encoding=int"
            return query_params

    class _BaseGetTableSpec:
        def __hash__(self):  # pragma: NO COVER
            return NotImplementedError("__hash__ must be implemented.")

        __REQUIRED_FIELDS_DEFAULT_VALUES: Dict[str, Any] = {}

        @classmethod
        def _get_unset_required_fields(cls, message_dict):
            return {
                k: v
                for k, v in cls.__REQUIRED_FIELDS_DEFAULT_VALUES.items()
                if k not in message_dict
            }

        @staticmethod
        def _get_http_options():
            http_options: List[Dict[str, str]] = [
                {
                    "method": "get",
                    "uri": "/v1beta1/{name=projects/*/locations/*/datasets/*/tableSpecs/*}",
                },
            ]
            return http_options

        @staticmethod
        def _get_transcoded_request(http_options, request):
            pb_request = service.GetTableSpecRequest.pb(request)
            transcoded_request = path_template.transcode(http_options, pb_request)
            return transcoded_request

        @staticmethod
        def _get_query_params_json(transcoded_request):
            query_params = json.loads(
                json_format.MessageToJson(
                    transcoded_request["query_params"],
                    use_integers_for_enums=True,
                )
            )
            query_params.update(
                _BaseAutoMlRestTransport._BaseGetTableSpec._get_unset_required_fields(
                    query_params
                )
            )

            query_params["$alt"] = "json;enum-encoding=int"
            return query_params

    class _BaseImportData:
        def __hash__(self):  # pragma: NO COVER
            return NotImplementedError("__hash__ must be implemented.")

        __REQUIRED_FIELDS_DEFAULT_VALUES: Dict[str, Any] = {}

        @classmethod
        def _get_unset_required_fields(cls, message_dict):
            return {
                k: v
                for k, v in cls.__REQUIRED_FIELDS_DEFAULT_VALUES.items()
                if k not in message_dict
            }

        @staticmethod
        def _get_http_options():
            http_options: List[Dict[str, str]] = [
                {
                    "method": "post",
                    "uri": "/v1beta1/{name=projects/*/locations/*/datasets/*}:importData",
                    "body": "*",
                },
            ]
            return http_options

        @staticmethod
        def _get_transcoded_request(http_options, request):
            pb_request = service.ImportDataRequest.pb(request)
            transcoded_request = path_template.transcode(http_options, pb_request)
            return transcoded_request

        @staticmethod
        def _get_request_body_json(transcoded_request):
            # Jsonify the request body

            body = json_format.MessageToJson(
                transcoded_request["body"], use_integers_for_enums=True
            )
            return body

        @staticmethod
        def _get_query_params_json(transcoded_request):
            query_params = json.loads(
                json_format.MessageToJson(
                    transcoded_request["query_params"],
                    use_integers_for_enums=True,
                )
            )
            query_params.update(
                _BaseAutoMlRestTransport._BaseImportData._get_unset_required_fields(
                    query_params
                )
            )

            query_params["$alt"] = "json;enum-encoding=int"
            return query_params

    class _BaseListColumnSpecs:
        def __hash__(self):  # pragma: NO COVER
            return NotImplementedError("__hash__ must be implemented.")

        __REQUIRED_FIELDS_DEFAULT_VALUES: Dict[str, Any] = {}

        @classmethod
        def _get_unset_required_fields(cls, message_dict):
            return {
                k: v
                for k, v in cls.__REQUIRED_FIELDS_DEFAULT_VALUES.items()
                if k not in message_dict
            }

        @staticmethod
        def _get_http_options():
            http_options: List[Dict[str, str]] = [
                {
                    "method": "get",
                    "uri": "/v1beta1/{parent=projects/*/locations/*/datasets/*/tableSpecs/*}/columnSpecs",
                },
            ]
            return http_options

        @staticmethod
        def _get_transcoded_request(http_options, request):
            pb_request = service.ListColumnSpecsRequest.pb(request)
            transcoded_request = path_template.transcode(http_options, pb_request)
            return transcoded_request

        @staticmethod
        def _get_query_params_json(transcoded_request):
            query_params = json.loads(
                json_format.MessageToJson(
                    transcoded_request["query_params"],
                    use_integers_for_enums=True,
                )
            )
            query_params.update(
                _BaseAutoMlRestTransport._BaseListColumnSpecs._get_unset_required_fields(
                    query_params
                )
            )

            query_params["$alt"] = "json;enum-encoding=int"
            return query_params

    class _BaseListDatasets:
        def __hash__(self):  # pragma: NO COVER
            return NotImplementedError("__hash__ must be implemented.")

        __REQUIRED_FIELDS_DEFAULT_VALUES: Dict[str, Any] = {}

        @classmethod
        def _get_unset_required_fields(cls, message_dict):
            return {
                k: v
                for k, v in cls.__REQUIRED_FIELDS_DEFAULT_VALUES.items()
                if k not in message_dict
            }

        @staticmethod
        def _get_http_options():
            http_options: List[Dict[str, str]] = [
                {
                    "method": "get",
                    "uri": "/v1beta1/{parent=projects/*/locations/*}/datasets",
                },
            ]
            return http_options

        @staticmethod
        def _get_transcoded_request(http_options, request):
            pb_request = service.ListDatasetsRequest.pb(request)
            transcoded_request = path_template.transcode(http_options, pb_request)
            return transcoded_request

        @staticmethod
        def _get_query_params_json(transcoded_request):
            query_params = json.loads(
                json_format.MessageToJson(
                    transcoded_request["query_params"],
                    use_integers_for_enums=True,
                )
            )
            query_params.update(
                _BaseAutoMlRestTransport._BaseListDatasets._get_unset_required_fields(
                    query_params
                )
            )

            query_params["$alt"] = "json;enum-encoding=int"
            return query_params

    class _BaseListModelEvaluations:
        def __hash__(self):  # pragma: NO COVER
            return NotImplementedError("__hash__ must be implemented.")

        __REQUIRED_FIELDS_DEFAULT_VALUES: Dict[str, Any] = {}

        @classmethod
        def _get_unset_required_fields(cls, message_dict):
            return {
                k: v
                for k, v in cls.__REQUIRED_FIELDS_DEFAULT_VALUES.items()
                if k not in message_dict
            }

        @staticmethod
        def _get_http_options():
            http_options: List[Dict[str, str]] = [
                {
                    "method": "get",
                    "uri": "/v1beta1/{parent=projects/*/locations/*/models/*}/modelEvaluations",
                },
            ]
            return http_options

        @staticmethod
        def _get_transcoded_request(http_options, request):
            pb_request = service.ListModelEvaluationsRequest.pb(request)
            transcoded_request = path_template.transcode(http_options, pb_request)
            return transcoded_request

        @staticmethod
        def _get_query_params_json(transcoded_request):
            query_params = json.loads(
                json_format.MessageToJson(
                    transcoded_request["query_params"],
                    use_integers_for_enums=True,
                )
            )
            query_params.update(
                _BaseAutoMlRestTransport._BaseListModelEvaluations._get_unset_required_fields(
                    query_params
                )
            )

            query_params["$alt"] = "json;enum-encoding=int"
            return query_params

    class _BaseListModels:
        def __hash__(self):  # pragma: NO COVER
            return NotImplementedError("__hash__ must be implemented.")

        __REQUIRED_FIELDS_DEFAULT_VALUES: Dict[str, Any] = {}

        @classmethod
        def _get_unset_required_fields(cls, message_dict):
            return {
                k: v
                for k, v in cls.__REQUIRED_FIELDS_DEFAULT_VALUES.items()
                if k not in message_dict
            }

        @staticmethod
        def _get_http_options():
            http_options: List[Dict[str, str]] = [
                {
                    "method": "get",
                    "uri": "/v1beta1/{parent=projects/*/locations/*}/models",
                },
            ]
            return http_options

        @staticmethod
        def _get_transcoded_request(http_options, request):
            pb_request = service.ListModelsRequest.pb(request)
            transcoded_request = path_template.transcode(http_options, pb_request)
            return transcoded_request

        @staticmethod
        def _get_query_params_json(transcoded_request):
            query_params = json.loads(
                json_format.MessageToJson(
                    transcoded_request["query_params"],
                    use_integers_for_enums=True,
                )
            )
            query_params.update(
                _BaseAutoMlRestTransport._BaseListModels._get_unset_required_fields(
                    query_params
                )
            )

            query_params["$alt"] = "json;enum-encoding=int"
            return query_params

    class _BaseListTableSpecs:
        def __hash__(self):  # pragma: NO COVER
            return NotImplementedError("__hash__ must be implemented.")

        __REQUIRED_FIELDS_DEFAULT_VALUES: Dict[str, Any] = {}

        @classmethod
        def _get_unset_required_fields(cls, message_dict):
            return {
                k: v
                for k, v in cls.__REQUIRED_FIELDS_DEFAULT_VALUES.items()
                if k not in message_dict
            }

        @staticmethod
        def _get_http_options():
            http_options: List[Dict[str, str]] = [
                {
                    "method": "get",
                    "uri": "/v1beta1/{parent=projects/*/locations/*/datasets/*}/tableSpecs",
                },
            ]
            return http_options

        @staticmethod
        def _get_transcoded_request(http_options, request):
            pb_request = service.ListTableSpecsRequest.pb(request)
            transcoded_request = path_template.transcode(http_options, pb_request)
            return transcoded_request

        @staticmethod
        def _get_query_params_json(transcoded_request):
            query_params = json.loads(
                json_format.MessageToJson(
                    transcoded_request["query_params"],
                    use_integers_for_enums=True,
                )
            )
            query_params.update(
                _BaseAutoMlRestTransport._BaseListTableSpecs._get_unset_required_fields(
                    query_params
                )
            )

            query_params["$alt"] = "json;enum-encoding=int"
            return query_params

    class _BaseUndeployModel:
        def __hash__(self):  # pragma: NO COVER
            return NotImplementedError("__hash__ must be implemented.")

        __REQUIRED_FIELDS_DEFAULT_VALUES: Dict[str, Any] = {}

        @classmethod
        def _get_unset_required_fields(cls, message_dict):
            return {
                k: v
                for k, v in cls.__REQUIRED_FIELDS_DEFAULT_VALUES.items()
                if k not in message_dict
            }

        @staticmethod
        def _get_http_options():
            http_options: List[Dict[str, str]] = [
                {
                    "method": "post",
                    "uri": "/v1beta1/{name=projects/*/locations/*/models/*}:undeploy",
                    "body": "*",
                },
            ]
            return http_options

        @staticmethod
        def _get_transcoded_request(http_options, request):
            pb_request = service.UndeployModelRequest.pb(request)
            transcoded_request = path_template.transcode(http_options, pb_request)
            return transcoded_request

        @staticmethod
        def _get_request_body_json(transcoded_request):
            # Jsonify the request body

            body = json_format.MessageToJson(
                transcoded_request["body"], use_integers_for_enums=True
            )
            return body

        @staticmethod
        def _get_query_params_json(transcoded_request):
            query_params = json.loads(
                json_format.MessageToJson(
                    transcoded_request["query_params"],
                    use_integers_for_enums=True,
                )
            )
            query_params.update(
                _BaseAutoMlRestTransport._BaseUndeployModel._get_unset_required_fields(
                    query_params
                )
            )

            query_params["$alt"] = "json;enum-encoding=int"
            return query_params

    class _BaseUpdateColumnSpec:
        def __hash__(self):  # pragma: NO COVER
            return NotImplementedError("__hash__ must be implemented.")

        __REQUIRED_FIELDS_DEFAULT_VALUES: Dict[str, Any] = {}

        @classmethod
        def _get_unset_required_fields(cls, message_dict):
            return {
                k: v
                for k, v in cls.__REQUIRED_FIELDS_DEFAULT_VALUES.items()
                if k not in message_dict
            }

        @staticmethod
        def _get_http_options():
            http_options: List[Dict[str, str]] = [
                {
                    "method": "patch",
                    "uri": "/v1beta1/{column_spec.name=projects/*/locations/*/datasets/*/tableSpecs/*/columnSpecs/*}",
                    "body": "column_spec",
                },
            ]
            return http_options

        @staticmethod
        def _get_transcoded_request(http_options, request):
            pb_request = service.UpdateColumnSpecRequest.pb(request)
            transcoded_request = path_template.transcode(http_options, pb_request)
            return transcoded_request

        @staticmethod
        def _get_request_body_json(transcoded_request):
            # Jsonify the request body

            body = json_format.MessageToJson(
                transcoded_request["body"], use_integers_for_enums=True
            )
            return body

        @staticmethod
        def _get_query_params_json(transcoded_request):
            query_params = json.loads(
                json_format.MessageToJson(
                    transcoded_request["query_params"],
                    use_integers_for_enums=True,
                )
            )
            query_params.update(
                _BaseAutoMlRestTransport._BaseUpdateColumnSpec._get_unset_required_fields(
                    query_params
                )
            )

            query_params["$alt"] = "json;enum-encoding=int"
            return query_params

    class _BaseUpdateDataset:
        def __hash__(self):  # pragma: NO COVER
            return NotImplementedError("__hash__ must be implemented.")

        __REQUIRED_FIELDS_DEFAULT_VALUES: Dict[str, Any] = {}

        @classmethod
        def _get_unset_required_fields(cls, message_dict):
            return {
                k: v
                for k, v in cls.__REQUIRED_FIELDS_DEFAULT_VALUES.items()
                if k not in message_dict
            }

        @staticmethod
        def _get_http_options():
            http_options: List[Dict[str, str]] = [
                {
                    "method": "patch",
                    "uri": "/v1beta1/{dataset.name=projects/*/locations/*/datasets/*}",
                    "body": "dataset",
                },
            ]
            return http_options

        @staticmethod
        def _get_transcoded_request(http_options, request):
            pb_request = service.UpdateDatasetRequest.pb(request)
            transcoded_request = path_template.transcode(http_options, pb_request)
            return transcoded_request

        @staticmethod
        def _get_request_body_json(transcoded_request):
            # Jsonify the request body

            body = json_format.MessageToJson(
                transcoded_request["body"], use_integers_for_enums=True
            )
            return body

        @staticmethod
        def _get_query_params_json(transcoded_request):
            query_params = json.loads(
                json_format.MessageToJson(
                    transcoded_request["query_params"],
                    use_integers_for_enums=True,
                )
            )
            query_params.update(
                _BaseAutoMlRestTransport._BaseUpdateDataset._get_unset_required_fields(
                    query_params
                )
            )

            query_params["$alt"] = "json;enum-encoding=int"
            return query_params

    class _BaseUpdateTableSpec:
        def __hash__(self):  # pragma: NO COVER
            return NotImplementedError("__hash__ must be implemented.")

        __REQUIRED_FIELDS_DEFAULT_VALUES: Dict[str, Any] = {}

        @classmethod
        def _get_unset_required_fields(cls, message_dict):
            return {
                k: v
                for k, v in cls.__REQUIRED_FIELDS_DEFAULT_VALUES.items()
                if k not in message_dict
            }

        @staticmethod
        def _get_http_options():
            http_options: List[Dict[str, str]] = [
                {
                    "method": "patch",
                    "uri": "/v1beta1/{table_spec.name=projects/*/locations/*/datasets/*/tableSpecs/*}",
                    "body": "table_spec",
                },
            ]
            return http_options

        @staticmethod
        def _get_transcoded_request(http_options, request):
            pb_request = service.UpdateTableSpecRequest.pb(request)
            transcoded_request = path_template.transcode(http_options, pb_request)
            return transcoded_request

        @staticmethod
        def _get_request_body_json(transcoded_request):
            # Jsonify the request body

            body = json_format.MessageToJson(
                transcoded_request["body"], use_integers_for_enums=True
            )
            return body

        @staticmethod
        def _get_query_params_json(transcoded_request):
            query_params = json.loads(
                json_format.MessageToJson(
                    transcoded_request["query_params"],
                    use_integers_for_enums=True,
                )
            )
            query_params.update(
                _BaseAutoMlRestTransport._BaseUpdateTableSpec._get_unset_required_fields(
                    query_params
                )
            )

            query_params["$alt"] = "json;enum-encoding=int"
            return query_params


__all__ = ("_BaseAutoMlRestTransport",)



# -*- coding: utf-8 -*-
# Copyright 2024 Google LLC
#
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
#     http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.
#
from __future__ import annotations

from typing import MutableMapping, MutableSequence

import proto  # type: ignore

from google.cloud.automl_v1beta1.types import classification

__protobuf__ = proto.module(
    package="google.cloud.automl.v1beta1",
    manifest={
        "TextSentimentAnnotation",
        "TextSentimentEvaluationMetrics",
    },
)


class TextSentimentAnnotation(proto.Message):
    r"""Contains annotation details specific to text sentiment.

    Attributes:
        sentiment (int):
            Output only. The sentiment with the semantic, as given to
            the
            [AutoMl.ImportData][google.cloud.automl.v1beta1.AutoMl.ImportData]
            when populating the dataset from which the model used for
            the prediction had been trained. The sentiment values are
            between 0 and
            Dataset.text_sentiment_dataset_metadata.sentiment_max
            (inclusive), with higher value meaning more positive
            sentiment. They are completely relative, i.e. 0 means least
            positive sentiment and sentiment_max means the most positive
            from the sentiments present in the train data. Therefore
            e.g. if train data had only negative sentiment, then
            sentiment_max, would be still negative (although least
            negative). The sentiment shouldn't be confused with "score"
            or "magnitude" from the previous Natural Language Sentiment
            Analysis API.
    """

    sentiment: int = proto.Field(
        proto.INT32,
        number=1,
    )


class TextSentimentEvaluationMetrics(proto.Message):
    r"""Model evaluation metrics for text sentiment problems.

    Attributes:
        precision (float):
            Output only. Precision.
        recall (float):
            Output only. Recall.
        f1_score (float):
            Output only. The harmonic mean of recall and
            precision.
        mean_absolute_error (float):
            Output only. Mean absolute error. Only set
            for the overall model evaluation, not for
            evaluation of a single annotation spec.
        mean_squared_error (float):
            Output only. Mean squared error. Only set for
            the overall model evaluation, not for evaluation
            of a single annotation spec.
        linear_kappa (float):
            Output only. Linear weighted kappa. Only set
            for the overall model evaluation, not for
            evaluation of a single annotation spec.
        quadratic_kappa (float):
            Output only. Quadratic weighted kappa. Only
            set for the overall model evaluation, not for
            evaluation of a single annotation spec.
        confusion_matrix (google.cloud.automl_v1beta1.types.ClassificationEvaluationMetrics.ConfusionMatrix):
            Output only. Confusion matrix of the
            evaluation. Only set for the overall model
            evaluation, not for evaluation of a single
            annotation spec.
        annotation_spec_id (MutableSequence[str]):
            Output only. The annotation spec ids used for
            this evaluation. Deprecated .
    """

    precision: float = proto.Field(
        proto.FLOAT,
        number=1,
    )
    recall: float = proto.Field(
        proto.FLOAT,
        number=2,
    )
    f1_score: float = proto.Field(
        proto.FLOAT,
        number=3,
    )
    mean_absolute_error: float = proto.Field(
        proto.FLOAT,
        number=4,
    )
    mean_squared_error: float = proto.Field(
        proto.FLOAT,
        number=5,
    )
    linear_kappa: float = proto.Field(
        proto.FLOAT,
        number=6,
    )
    quadratic_kappa: float = proto.Field(
        proto.FLOAT,
        number=7,
    )
    confusion_matrix: classification.ClassificationEvaluationMetrics.ConfusionMatrix = (
        proto.Field(
            proto.MESSAGE,
            number=8,
            message=classification.ClassificationEvaluationMetrics.ConfusionMatrix,
        )
    )
    annotation_spec_id: MutableSequence[str] = proto.RepeatedField(
        proto.STRING,
        number=9,
    )


__all__ = tuple(sorted(__protobuf__.manifest))



# -*- coding: utf-8 -*-
# Copyright 2024 Google LLC
#
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
#     http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.
#
from __future__ import annotations

from typing import MutableMapping, MutableSequence

import proto  # type: ignore

__protobuf__ = proto.module(
    package="google.cloud.automl.v1beta1",
    manifest={
        "VideoClassificationDatasetMetadata",
        "VideoObjectTrackingDatasetMetadata",
        "VideoClassificationModelMetadata",
        "VideoObjectTrackingModelMetadata",
    },
)


class VideoClassificationDatasetMetadata(proto.Message):
    r"""Dataset metadata specific to video classification.
    All Video Classification datasets are treated as multi label.

    """


class VideoObjectTrackingDatasetMetadata(proto.Message):
    r"""Dataset metadata specific to video object tracking."""


class VideoClassificationModelMetadata(proto.Message):
    r"""Model metadata specific to video classification."""


class VideoObjectTrackingModelMetadata(proto.Message):
    r"""Model metadata specific to video object tracking."""


__all__ = tuple(sorted(__protobuf__.manifest))



# -*- coding: utf-8 -*-
# Copyright 2024 Google LLC
#
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
#     http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.
#
from typing import Callable, Dict, Optional, Sequence, Tuple, Union
import warnings

from google.api_core import gapic_v1, grpc_helpers, operations_v1
import google.auth  # type: ignore
from google.auth import credentials as ga_credentials  # type: ignore
from google.auth.transport.grpc import SslCredentials  # type: ignore
from google.longrunning import operations_pb2  # type: ignore
import grpc  # type: ignore

from google.cloud.automl_v1.types import annotation_spec
from google.cloud.automl_v1.types import dataset
from google.cloud.automl_v1.types import dataset as gca_dataset
from google.cloud.automl_v1.types import model
from google.cloud.automl_v1.types import model as gca_model
from google.cloud.automl_v1.types import model_evaluation, service

from .base import DEFAULT_CLIENT_INFO, AutoMlTransport


class AutoMlGrpcTransport(AutoMlTransport):
    """gRPC backend transport for AutoMl.

    AutoML Server API.

    The resource names are assigned by the server. The server never
    reuses names that it has created after the resources with those
    names are deleted.

    An ID of a resource is the last element of the item's resource name.
    For
    ``projects/{project_id}/locations/{location_id}/datasets/{dataset_id}``,
    then the id for the item is ``{dataset_id}``.

    Currently the only supported ``location_id`` is "us-central1".

    On any input that is documented to expect a string parameter in
    snake_case or dash-case, either of those cases is accepted.

    This class defines the same methods as the primary client, so the
    primary client can load the underlying transport implementation
    and call it.

    It sends protocol buffers over the wire using gRPC (which is built on
    top of HTTP/2); the ``grpcio`` package must be installed.
    """

    _stubs: Dict[str, Callable]

    def __init__(
        self,
        *,
        host: str = "automl.googleapis.com",
        credentials: Optional[ga_credentials.Credentials] = None,
        credentials_file: Optional[str] = None,
        scopes: Optional[Sequence[str]] = None,
        channel: Optional[Union[grpc.Channel, Callable[..., grpc.Channel]]] = None,
        api_mtls_endpoint: Optional[str] = None,
        client_cert_source: Optional[Callable[[], Tuple[bytes, bytes]]] = None,
        ssl_channel_credentials: Optional[grpc.ChannelCredentials] = None,
        client_cert_source_for_mtls: Optional[Callable[[], Tuple[bytes, bytes]]] = None,
        quota_project_id: Optional[str] = None,
        client_info: gapic_v1.client_info.ClientInfo = DEFAULT_CLIENT_INFO,
        always_use_jwt_access: Optional[bool] = False,
        api_audience: Optional[str] = None,
    ) -> None:
        """Instantiate the transport.

        Args:
            host (Optional[str]):
                 The hostname to connect to (default: 'automl.googleapis.com').
            credentials (Optional[google.auth.credentials.Credentials]): The
                authorization credentials to attach to requests. These
                credentials identify the application to the service; if none
                are specified, the client will attempt to ascertain the
                credentials from the environment.
                This argument is ignored if a ``channel`` instance is provided.
            credentials_file (Optional[str]): A file with credentials that can
                be loaded with :func:`google.auth.load_credentials_from_file`.
                This argument is ignored if a ``channel`` instance is provided.
            scopes (Optional(Sequence[str])): A list of scopes. This argument is
                ignored if a ``channel`` instance is provided.
            channel (Optional[Union[grpc.Channel, Callable[..., grpc.Channel]]]):
                A ``Channel`` instance through which to make calls, or a Callable
                that constructs and returns one. If set to None, ``self.create_channel``
                is used to create the channel. If a Callable is given, it will be called
                with the same arguments as used in ``self.create_channel``.
            api_mtls_endpoint (Optional[str]): Deprecated. The mutual TLS endpoint.
                If provided, it overrides the ``host`` argument and tries to create
                a mutual TLS channel with client SSL credentials from
                ``client_cert_source`` or application default SSL credentials.
            client_cert_source (Optional[Callable[[], Tuple[bytes, bytes]]]):
                Deprecated. A callback to provide client SSL certificate bytes and
                private key bytes, both in PEM format. It is ignored if
                ``api_mtls_endpoint`` is None.
            ssl_channel_credentials (grpc.ChannelCredentials): SSL credentials
                for the grpc channel. It is ignored if a ``channel`` instance is provided.
            client_cert_source_for_mtls (Optional[Callable[[], Tuple[bytes, bytes]]]):
                A callback to provide client certificate bytes and private key bytes,
                both in PEM format. It is used to configure a mutual TLS channel. It is
                ignored if a ``channel`` instance or ``ssl_channel_credentials`` is provided.
            quota_project_id (Optional[str]): An optional project to use for billing
                and quota.
            client_info (google.api_core.gapic_v1.client_info.ClientInfo):
                The client info used to send a user-agent string along with
                API requests. If ``None``, then default info will be used.
                Generally, you only need to set this if you're developing
                your own client library.
            always_use_jwt_access (Optional[bool]): Whether self signed JWT should
                be used for service account credentials.

        Raises:
          google.auth.exceptions.MutualTLSChannelError: If mutual TLS transport
              creation failed for any reason.
          google.api_core.exceptions.DuplicateCredentialArgs: If both ``credentials``
              and ``credentials_file`` are passed.
        """
        self._grpc_channel = None
        self._ssl_channel_credentials = ssl_channel_credentials
        self._stubs: Dict[str, Callable] = {}
        self._operations_client: Optional[operations_v1.OperationsClient] = None

        if api_mtls_endpoint:
            warnings.warn("api_mtls_endpoint is deprecated", DeprecationWarning)
        if client_cert_source:
            warnings.warn("client_cert_source is deprecated", DeprecationWarning)

        if isinstance(channel, grpc.Channel):
            # Ignore credentials if a channel was passed.
            credentials = None
            self._ignore_credentials = True
            # If a channel was explicitly provided, set it.
            self._grpc_channel = channel
            self._ssl_channel_credentials = None

        else:
            if api_mtls_endpoint:
                host = api_mtls_endpoint

                # Create SSL credentials with client_cert_source or application
                # default SSL credentials.
                if client_cert_source:
                    cert, key = client_cert_source()
                    self._ssl_channel_credentials = grpc.ssl_channel_credentials(
                        certificate_chain=cert, private_key=key
                    )
                else:
                    self._ssl_channel_credentials = SslCredentials().ssl_credentials

            else:
                if client_cert_source_for_mtls and not ssl_channel_credentials:
                    cert, key = client_cert_source_for_mtls()
                    self._ssl_channel_credentials = grpc.ssl_channel_credentials(
                        certificate_chain=cert, private_key=key
                    )

        # The base transport sets the host, credentials and scopes
        super().__init__(
            host=host,
            credentials=credentials,
            credentials_file=credentials_file,
            scopes=scopes,
            quota_project_id=quota_project_id,
            client_info=client_info,
            always_use_jwt_access=always_use_jwt_access,
            api_audience=api_audience,
        )

        if not self._grpc_channel:
            # initialize with the provided callable or the default channel
            channel_init = channel or type(self).create_channel
            self._grpc_channel = channel_init(
                self._host,
                # use the credentials which are saved
                credentials=self._credentials,
                # Set ``credentials_file`` to ``None`` here as
                # the credentials that we saved earlier should be used.
                credentials_file=None,
                scopes=self._scopes,
                ssl_credentials=self._ssl_channel_credentials,
                quota_project_id=quota_project_id,
                options=[
                    ("grpc.max_send_message_length", -1),
                    ("grpc.max_receive_message_length", -1),
                ],
            )

        # Wrap messages. This must be done after self._grpc_channel exists
        self._prep_wrapped_messages(client_info)

    @classmethod
    def create_channel(
        cls,
        host: str = "automl.googleapis.com",
        credentials: Optional[ga_credentials.Credentials] = None,
        credentials_file: Optional[str] = None,
        scopes: Optional[Sequence[str]] = None,
        quota_project_id: Optional[str] = None,
        **kwargs,
    ) -> grpc.Channel:
        """Create and return a gRPC channel object.
        Args:
            host (Optional[str]): The host for the channel to use.
            credentials (Optional[~.Credentials]): The
                authorization credentials to attach to requests. These
                credentials identify this application to the service. If
                none are specified, the client will attempt to ascertain
                the credentials from the environment.
            credentials_file (Optional[str]): A file with credentials that can
                be loaded with :func:`google.auth.load_credentials_from_file`.
                This argument is mutually exclusive with credentials.
            scopes (Optional[Sequence[str]]): A optional list of scopes needed for this
                service. These are only used when credentials are not specified and
                are passed to :func:`google.auth.default`.
            quota_project_id (Optional[str]): An optional project to use for billing
                and quota.
            kwargs (Optional[dict]): Keyword arguments, which are passed to the
                channel creation.
        Returns:
            grpc.Channel: A gRPC channel object.

        Raises:
            google.api_core.exceptions.DuplicateCredentialArgs: If both ``credentials``
              and ``credentials_file`` are passed.
        """

        return grpc_helpers.create_channel(
            host,
            credentials=credentials,
            credentials_file=credentials_file,
            quota_project_id=quota_project_id,
            default_scopes=cls.AUTH_SCOPES,
            scopes=scopes,
            default_host=cls.DEFAULT_HOST,
            **kwargs,
        )

    @property
    def grpc_channel(self) -> grpc.Channel:
        """Return the channel designed to connect to this service."""
        return self._grpc_channel

    @property
    def operations_client(self) -> operations_v1.OperationsClient:
        """Create the client designed to process long-running operations.

        This property caches on the instance; repeated calls return the same
        client.
        """
        # Quick check: Only create a new client if we do not already have one.
        if self._operations_client is None:
            self._operations_client = operations_v1.OperationsClient(self.grpc_channel)

        # Return the client from cache.
        return self._operations_client

    @property
    def create_dataset(
        self,
    ) -> Callable[[service.CreateDatasetRequest], operations_pb2.Operation]:
        r"""Return a callable for the create dataset method over gRPC.

        Creates a dataset.

        Returns:
            Callable[[~.CreateDatasetRequest],
                    ~.Operation]:
                A function that, when called, will call the underlying RPC
                on the server.
        """
        # Generate a "stub function" on-the-fly which will actually make
        # the request.
        # gRPC handles serialization and deserialization, so we just need
        # to pass in the functions for each.
        if "create_dataset" not in self._stubs:
            self._stubs["create_dataset"] = self.grpc_channel.unary_unary(
                "/google.cloud.automl.v1.AutoMl/CreateDataset",
                request_serializer=service.CreateDatasetRequest.serialize,
                response_deserializer=operations_pb2.Operation.FromString,
            )
        return self._stubs["create_dataset"]

    @property
    def get_dataset(self) -> Callable[[service.GetDatasetRequest], dataset.Dataset]:
        r"""Return a callable for the get dataset method over gRPC.

        Gets a dataset.

        Returns:
            Callable[[~.GetDatasetRequest],
                    ~.Dataset]:
                A function that, when called, will call the underlying RPC
                on the server.
        """
        # Generate a "stub function" on-the-fly which will actually make
        # the request.
        # gRPC handles serialization and deserialization, so we just need
        # to pass in the functions for each.
        if "get_dataset" not in self._stubs:
            self._stubs["get_dataset"] = self.grpc_channel.unary_unary(
                "/google.cloud.automl.v1.AutoMl/GetDataset",
                request_serializer=service.GetDatasetRequest.serialize,
                response_deserializer=dataset.Dataset.deserialize,
            )
        return self._stubs["get_dataset"]

    @property
    def list_datasets(
        self,
    ) -> Callable[[service.ListDatasetsRequest], service.ListDatasetsResponse]:
        r"""Return a callable for the list datasets method over gRPC.

        Lists datasets in a project.

        Returns:
            Callable[[~.ListDatasetsRequest],
                    ~.ListDatasetsResponse]:
                A function that, when called, will call the underlying RPC
                on the server.
        """
        # Generate a "stub function" on-the-fly which will actually make
        # the request.
        # gRPC handles serialization and deserialization, so we just need
        # to pass in the functions for each.
        if "list_datasets" not in self._stubs:
            self._stubs["list_datasets"] = self.grpc_channel.unary_unary(
                "/google.cloud.automl.v1.AutoMl/ListDatasets",
                request_serializer=service.ListDatasetsRequest.serialize,
                response_deserializer=service.ListDatasetsResponse.deserialize,
            )
        return self._stubs["list_datasets"]

    @property
    def update_dataset(
        self,
    ) -> Callable[[service.UpdateDatasetRequest], gca_dataset.Dataset]:
        r"""Return a callable for the update dataset method over gRPC.

        Updates a dataset.

        Returns:
            Callable[[~.UpdateDatasetRequest],
                    ~.Dataset]:
                A function that, when called, will call the underlying RPC
                on the server.
        """
        # Generate a "stub function" on-the-fly which will actually make
        # the request.
        # gRPC handles serialization and deserialization, so we just need
        # to pass in the functions for each.
        if "update_dataset" not in self._stubs:
            self._stubs["update_dataset"] = self.grpc_channel.unary_unary(
                "/google.cloud.automl.v1.AutoMl/UpdateDataset",
                request_serializer=service.UpdateDatasetRequest.serialize,
                response_deserializer=gca_dataset.Dataset.deserialize,
            )
        return self._stubs["update_dataset"]

    @property
    def delete_dataset(
        self,
    ) -> Callable[[service.DeleteDatasetRequest], operations_pb2.Operation]:
        r"""Return a callable for the delete dataset method over gRPC.

        Deletes a dataset and all of its contents. Returns empty
        response in the
        [response][google.longrunning.Operation.response] field when it
        completes, and ``delete_details`` in the
        [metadata][google.longrunning.Operation.metadata] field.

        Returns:
            Callable[[~.DeleteDatasetRequest],
                    ~.Operation]:
                A function that, when called, will call the underlying RPC
                on the server.
        """
        # Generate a "stub function" on-the-fly which will actually make
        # the request.
        # gRPC handles serialization and deserialization, so we just need
        # to pass in the functions for each.
        if "delete_dataset" not in self._stubs:
            self._stubs["delete_dataset"] = self.grpc_channel.unary_unary(
                "/google.cloud.automl.v1.AutoMl/DeleteDataset",
                request_serializer=service.DeleteDatasetRequest.serialize,
                response_deserializer=operations_pb2.Operation.FromString,
            )
        return self._stubs["delete_dataset"]

    @property
    def import_data(
        self,
    ) -> Callable[[service.ImportDataRequest], operations_pb2.Operation]:
        r"""Return a callable for the import data method over gRPC.

        Imports data into a dataset. For Tables this method can only be
        called on an empty Dataset.

        For Tables:

        -  A
           [schema_inference_version][google.cloud.automl.v1.InputConfig.params]
           parameter must be explicitly set. Returns an empty response
           in the [response][google.longrunning.Operation.response]
           field when it completes.

        Returns:
            Callable[[~.ImportDataRequest],
                    ~.Operation]:
                A function that, when called, will call the underlying RPC
                on the server.
        """
        # Generate a "stub function" on-the-fly which will actually make
        # the request.
        # gRPC handles serialization and deserialization, so we just need
        # to pass in the functions for each.
        if "import_data" not in self._stubs:
            self._stubs["import_data"] = self.grpc_channel.unary_unary(
                "/google.cloud.automl.v1.AutoMl/ImportData",
                request_serializer=service.ImportDataRequest.serialize,
                response_deserializer=operations_pb2.Operation.FromString,
            )
        return self._stubs["import_data"]

    @property
    def export_data(
        self,
    ) -> Callable[[service.ExportDataRequest], operations_pb2.Operation]:
        r"""Return a callable for the export data method over gRPC.

        Exports dataset's data to the provided output location. Returns
        an empty response in the
        [response][google.longrunning.Operation.response] field when it
        completes.

        Returns:
            Callable[[~.ExportDataRequest],
                    ~.Operation]:
                A function that, when called, will call the underlying RPC
                on the server.
        """
        # Generate a "stub function" on-the-fly which will actually make
        # the request.
        # gRPC handles serialization and deserialization, so we just need
        # to pass in the functions for each.
        if "export_data" not in self._stubs:
            self._stubs["export_data"] = self.grpc_channel.unary_unary(
                "/google.cloud.automl.v1.AutoMl/ExportData",
                request_serializer=service.ExportDataRequest.serialize,
                response_deserializer=operations_pb2.Operation.FromString,
            )
        return self._stubs["export_data"]

    @property
    def get_annotation_spec(
        self,
    ) -> Callable[[service.GetAnnotationSpecRequest], annotation_spec.AnnotationSpec]:
        r"""Return a callable for the get annotation spec method over gRPC.

        Gets an annotation spec.

        Returns:
            Callable[[~.GetAnnotationSpecRequest],
                    ~.AnnotationSpec]:
                A function that, when called, will call the underlying RPC
                on the server.
        """
        # Generate a "stub function" on-the-fly which will actually make
        # the request.
        # gRPC handles serialization and deserialization, so we just need
        # to pass in the functions for each.
        if "get_annotation_spec" not in self._stubs:
            self._stubs["get_annotation_spec"] = self.grpc_channel.unary_unary(
                "/google.cloud.automl.v1.AutoMl/GetAnnotationSpec",
                request_serializer=service.GetAnnotationSpecRequest.serialize,
                response_deserializer=annotation_spec.AnnotationSpec.deserialize,
            )
        return self._stubs["get_annotation_spec"]

    @property
    def create_model(
        self,
    ) -> Callable[[service.CreateModelRequest], operations_pb2.Operation]:
        r"""Return a callable for the create model method over gRPC.

        Creates a model. Returns a Model in the
        [response][google.longrunning.Operation.response] field when it
        completes. When you create a model, several model evaluations
        are created for it: a global evaluation, and one evaluation for
        each annotation spec.

        Returns:
            Callable[[~.CreateModelRequest],
                    ~.Operation]:
                A function that, when called, will call the underlying RPC
                on the server.
        """
        # Generate a "stub function" on-the-fly which will actually make
        # the request.
        # gRPC handles serialization and deserialization, so we just need
        # to pass in the functions for each.
        if "create_model" not in self._stubs:
            self._stubs["create_model"] = self.grpc_channel.unary_unary(
                "/google.cloud.automl.v1.AutoMl/CreateModel",
                request_serializer=service.CreateModelRequest.serialize,
                response_deserializer=operations_pb2.Operation.FromString,
            )
        return self._stubs["create_model"]

    @property
    def get_model(self) -> Callable[[service.GetModelRequest], model.Model]:
        r"""Return a callable for the get model method over gRPC.

        Gets a model.

        Returns:
            Callable[[~.GetModelRequest],
                    ~.Model]:
                A function that, when called, will call the underlying RPC
                on the server.
        """
        # Generate a "stub function" on-the-fly which will actually make
        # the request.
        # gRPC handles serialization and deserialization, so we just need
        # to pass in the functions for each.
        if "get_model" not in self._stubs:
            self._stubs["get_model"] = self.grpc_channel.unary_unary(
                "/google.cloud.automl.v1.AutoMl/GetModel",
                request_serializer=service.GetModelRequest.serialize,
                response_deserializer=model.Model.deserialize,
            )
        return self._stubs["get_model"]

    @property
    def list_models(
        self,
    ) -> Callable[[service.ListModelsRequest], service.ListModelsResponse]:
        r"""Return a callable for the list models method over gRPC.

        Lists models.

        Returns:
            Callable[[~.ListModelsRequest],
                    ~.ListModelsResponse]:
                A function that, when called, will call the underlying RPC
                on the server.
        """
        # Generate a "stub function" on-the-fly which will actually make
        # the request.
        # gRPC handles serialization and deserialization, so we just need
        # to pass in the functions for each.
        if "list_models" not in self._stubs:
            self._stubs["list_models"] = self.grpc_channel.unary_unary(
                "/google.cloud.automl.v1.AutoMl/ListModels",
                request_serializer=service.ListModelsRequest.serialize,
                response_deserializer=service.ListModelsResponse.deserialize,
            )
        return self._stubs["list_models"]

    @property
    def delete_model(
        self,
    ) -> Callable[[service.DeleteModelRequest], operations_pb2.Operation]:
        r"""Return a callable for the delete model method over gRPC.

        Deletes a model. Returns ``google.protobuf.Empty`` in the
        [response][google.longrunning.Operation.response] field when it
        completes, and ``delete_details`` in the
        [metadata][google.longrunning.Operation.metadata] field.

        Returns:
            Callable[[~.DeleteModelRequest],
                    ~.Operation]:
                A function that, when called, will call the underlying RPC
                on the server.
        """
        # Generate a "stub function" on-the-fly which will actually make
        # the request.
        # gRPC handles serialization and deserialization, so we just need
        # to pass in the functions for each.
        if "delete_model" not in self._stubs:
            self._stubs["delete_model"] = self.grpc_channel.unary_unary(
                "/google.cloud.automl.v1.AutoMl/DeleteModel",
                request_serializer=service.DeleteModelRequest.serialize,
                response_deserializer=operations_pb2.Operation.FromString,
            )
        return self._stubs["delete_model"]

    @property
    def update_model(self) -> Callable[[service.UpdateModelRequest], gca_model.Model]:
        r"""Return a callable for the update model method over gRPC.

        Updates a model.

        Returns:
            Callable[[~.UpdateModelRequest],
                    ~.Model]:
                A function that, when called, will call the underlying RPC
                on the server.
        """
        # Generate a "stub function" on-the-fly which will actually make
        # the request.
        # gRPC handles serialization and deserialization, so we just need
        # to pass in the functions for each.
        if "update_model" not in self._stubs:
            self._stubs["update_model"] = self.grpc_channel.unary_unary(
                "/google.cloud.automl.v1.AutoMl/UpdateModel",
                request_serializer=service.UpdateModelRequest.serialize,
                response_deserializer=gca_model.Model.deserialize,
            )
        return self._stubs["update_model"]

    @property
    def deploy_model(
        self,
    ) -> Callable[[service.DeployModelRequest], operations_pb2.Operation]:
        r"""Return a callable for the deploy model method over gRPC.

        Deploys a model. If a model is already deployed, deploying it
        with the same parameters has no effect. Deploying with different
        parametrs (as e.g. changing
        [node_number][google.cloud.automl.v1p1beta.ImageObjectDetectionModelDeploymentMetadata.node_number])
        will reset the deployment state without pausing the model's
        availability.

        Only applicable for Text Classification, Image Object Detection
        , Tables, and Image Segmentation; all other domains manage
        deployment automatically.

        Returns an empty response in the
        [response][google.longrunning.Operation.response] field when it
        completes.

        Returns:
            Callable[[~.DeployModelRequest],
                    ~.Operation]:
                A function that, when called, will call the underlying RPC
                on the server.
        """
        # Generate a "stub function" on-the-fly which will actually make
        # the request.
        # gRPC handles serialization and deserialization, so we just need
        # to pass in the functions for each.
        if "deploy_model" not in self._stubs:
            self._stubs["deploy_model"] = self.grpc_channel.unary_unary(
                "/google.cloud.automl.v1.AutoMl/DeployModel",
                request_serializer=service.DeployModelRequest.serialize,
                response_deserializer=operations_pb2.Operation.FromString,
            )
        return self._stubs["deploy_model"]

    @property
    def undeploy_model(
        self,
    ) -> Callable[[service.UndeployModelRequest], operations_pb2.Operation]:
        r"""Return a callable for the undeploy model method over gRPC.

        Undeploys a model. If the model is not deployed this method has
        no effect.

        Only applicable for Text Classification, Image Object Detection
        and Tables; all other domains manage deployment automatically.

        Returns an empty response in the
        [response][google.longrunning.Operation.response] field when it
        completes.

        Returns:
            Callable[[~.UndeployModelRequest],
                    ~.Operation]:
                A function that, when called, will call the underlying RPC
                on the server.
        """
        # Generate a "stub function" on-the-fly which will actually make
        # the request.
        # gRPC handles serialization and deserialization, so we just need
        # to pass in the functions for each.
        if "undeploy_model" not in self._stubs:
            self._stubs["undeploy_model"] = self.grpc_channel.unary_unary(
                "/google.cloud.automl.v1.AutoMl/UndeployModel",
                request_serializer=service.UndeployModelRequest.serialize,
                response_deserializer=operations_pb2.Operation.FromString,
            )
        return self._stubs["undeploy_model"]

    @property
    def export_model(
        self,
    ) -> Callable[[service.ExportModelRequest], operations_pb2.Operation]:
        r"""Return a callable for the export model method over gRPC.

        Exports a trained, "export-able", model to a user specified
        Google Cloud Storage location. A model is considered export-able
        if and only if it has an export format defined for it in
        [ModelExportOutputConfig][google.cloud.automl.v1.ModelExportOutputConfig].

        Returns an empty response in the
        [response][google.longrunning.Operation.response] field when it
        completes.

        Returns:
            Callable[[~.ExportModelRequest],
                    ~.Operation]:
                A function that, when called, will call the underlying RPC
                on the server.
        """
        # Generate a "stub function" on-the-fly which will actually make
        # the request.
        # gRPC handles serialization and deserialization, so we just need
        # to pass in the functions for each.
        if "export_model" not in self._stubs:
            self._stubs["export_model"] = self.grpc_channel.unary_unary(
                "/google.cloud.automl.v1.AutoMl/ExportModel",
                request_serializer=service.ExportModelRequest.serialize,
                response_deserializer=operations_pb2.Operation.FromString,
            )
        return self._stubs["export_model"]

    @property
    def get_model_evaluation(
        self,
    ) -> Callable[
        [service.GetModelEvaluationRequest], model_evaluation.ModelEvaluation
    ]:
        r"""Return a callable for the get model evaluation method over gRPC.

        Gets a model evaluation.

        Returns:
            Callable[[~.GetModelEvaluationRequest],
                    ~.ModelEvaluation]:
                A function that, when called, will call the underlying RPC
                on the server.
        """
        # Generate a "stub function" on-the-fly which will actually make
        # the request.
        # gRPC handles serialization and deserialization, so we just need
        # to pass in the functions for each.
        if "get_model_evaluation" not in self._stubs:
            self._stubs["get_model_evaluation"] = self.grpc_channel.unary_unary(
                "/google.cloud.automl.v1.AutoMl/GetModelEvaluation",
                request_serializer=service.GetModelEvaluationRequest.serialize,
                response_deserializer=model_evaluation.ModelEvaluation.deserialize,
            )
        return self._stubs["get_model_evaluation"]

    @property
    def list_model_evaluations(
        self,
    ) -> Callable[
        [service.ListModelEvaluationsRequest], service.ListModelEvaluationsResponse
    ]:
        r"""Return a callable for the list model evaluations method over gRPC.

        Lists model evaluations.

        Returns:
            Callable[[~.ListModelEvaluationsRequest],
                    ~.ListModelEvaluationsResponse]:
                A function that, when called, will call the underlying RPC
                on the server.
        """
        # Generate a "stub function" on-the-fly which will actually make
        # the request.
        # gRPC handles serialization and deserialization, so we just need
        # to pass in the functions for each.
        if "list_model_evaluations" not in self._stubs:
            self._stubs["list_model_evaluations"] = self.grpc_channel.unary_unary(
                "/google.cloud.automl.v1.AutoMl/ListModelEvaluations",
                request_serializer=service.ListModelEvaluationsRequest.serialize,
                response_deserializer=service.ListModelEvaluationsResponse.deserialize,
            )
        return self._stubs["list_model_evaluations"]

    def close(self):
        self.grpc_channel.close()

    @property
    def kind(self) -> str:
        return "grpc"


__all__ = ("AutoMlGrpcTransport",)



# -*- coding: utf-8 -*-
# Copyright 2024 Google LLC
#
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
#     http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.
#
from google.cloud.automl import gapic_version as package_version

__version__ = package_version.__version__


from google.cloud.automl_v1.services.auto_ml.async_client import AutoMlAsyncClient
from google.cloud.automl_v1.services.auto_ml.client import AutoMlClient
from google.cloud.automl_v1.services.prediction_service.async_client import (
    PredictionServiceAsyncClient,
)
from google.cloud.automl_v1.services.prediction_service.client import (
    PredictionServiceClient,
)
from google.cloud.automl_v1.types.annotation_payload import AnnotationPayload
from google.cloud.automl_v1.types.annotation_spec import AnnotationSpec
from google.cloud.automl_v1.types.classification import (
    ClassificationAnnotation,
    ClassificationEvaluationMetrics,
    ClassificationType,
)
from google.cloud.automl_v1.types.data_items import (
    Document,
    DocumentDimensions,
    ExamplePayload,
    Image,
    TextSnippet,
)
from google.cloud.automl_v1.types.dataset import Dataset
from google.cloud.automl_v1.types.detection import (
    BoundingBoxMetricsEntry,
    ImageObjectDetectionAnnotation,
    ImageObjectDetectionEvaluationMetrics,
)
from google.cloud.automl_v1.types.geometry import BoundingPoly, NormalizedVertex
from google.cloud.automl_v1.types.image import (
    ImageClassificationDatasetMetadata,
    ImageClassificationModelDeploymentMetadata,
    ImageClassificationModelMetadata,
    ImageObjectDetectionDatasetMetadata,
    ImageObjectDetectionModelDeploymentMetadata,
    ImageObjectDetectionModelMetadata,
)
from google.cloud.automl_v1.types.io import (
    BatchPredictInputConfig,
    BatchPredictOutputConfig,
    DocumentInputConfig,
    GcsDestination,
    GcsSource,
    InputConfig,
    ModelExportOutputConfig,
    OutputConfig,
)
from google.cloud.automl_v1.types.model import Model
from google.cloud.automl_v1.types.model_evaluation import ModelEvaluation
from google.cloud.automl_v1.types.operations import (
    BatchPredictOperationMetadata,
    CreateDatasetOperationMetadata,
    CreateModelOperationMetadata,
    DeleteOperationMetadata,
    DeployModelOperationMetadata,
    ExportDataOperationMetadata,
    ExportModelOperationMetadata,
    ImportDataOperationMetadata,
    OperationMetadata,
    UndeployModelOperationMetadata,
)
from google.cloud.automl_v1.types.prediction_service import (
    BatchPredictRequest,
    BatchPredictResult,
    PredictRequest,
    PredictResponse,
)
from google.cloud.automl_v1.types.service import (
    CreateDatasetRequest,
    CreateModelRequest,
    DeleteDatasetRequest,
    DeleteModelRequest,
    DeployModelRequest,
    ExportDataRequest,
    ExportModelRequest,
    GetAnnotationSpecRequest,
    GetDatasetRequest,
    GetModelEvaluationRequest,
    GetModelRequest,
    ImportDataRequest,
    ListDatasetsRequest,
    ListDatasetsResponse,
    ListModelEvaluationsRequest,
    ListModelEvaluationsResponse,
    ListModelsRequest,
    ListModelsResponse,
    UndeployModelRequest,
    UpdateDatasetRequest,
    UpdateModelRequest,
)
from google.cloud.automl_v1.types.text import (
    TextClassificationDatasetMetadata,
    TextClassificationModelMetadata,
    TextExtractionDatasetMetadata,
    TextExtractionModelMetadata,
    TextSentimentDatasetMetadata,
    TextSentimentModelMetadata,
)
from google.cloud.automl_v1.types.text_extraction import (
    TextExtractionAnnotation,
    TextExtractionEvaluationMetrics,
)
from google.cloud.automl_v1.types.text_segment import TextSegment
from google.cloud.automl_v1.types.text_sentiment import (
    TextSentimentAnnotation,
    TextSentimentEvaluationMetrics,
)
from google.cloud.automl_v1.types.translation import (
    TranslationAnnotation,
    TranslationDatasetMetadata,
    TranslationEvaluationMetrics,
    TranslationModelMetadata,
)

__all__ = (
    "AutoMlClient",
    "AutoMlAsyncClient",
    "PredictionServiceClient",
    "PredictionServiceAsyncClient",
    "AnnotationPayload",
    "AnnotationSpec",
    "ClassificationAnnotation",
    "ClassificationEvaluationMetrics",
    "ClassificationType",
    "Document",
    "DocumentDimensions",
    "ExamplePayload",
    "Image",
    "TextSnippet",
    "Dataset",
    "BoundingBoxMetricsEntry",
    "ImageObjectDetectionAnnotation",
    "ImageObjectDetectionEvaluationMetrics",
    "BoundingPoly",
    "NormalizedVertex",
    "ImageClassificationDatasetMetadata",
    "ImageClassificationModelDeploymentMetadata",
    "ImageClassificationModelMetadata",
    "ImageObjectDetectionDatasetMetadata",
    "ImageObjectDetectionModelDeploymentMetadata",
    "ImageObjectDetectionModelMetadata",
    "BatchPredictInputConfig",
    "BatchPredictOutputConfig",
    "DocumentInputConfig",
    "GcsDestination",
    "GcsSource",
    "InputConfig",
    "ModelExportOutputConfig",
    "OutputConfig",
    "Model",
    "ModelEvaluation",
    "BatchPredictOperationMetadata",
    "CreateDatasetOperationMetadata",
    "CreateModelOperationMetadata",
    "DeleteOperationMetadata",
    "DeployModelOperationMetadata",
    "ExportDataOperationMetadata",
    "ExportModelOperationMetadata",
    "ImportDataOperationMetadata",
    "OperationMetadata",
    "UndeployModelOperationMetadata",
    "BatchPredictRequest",
    "BatchPredictResult",
    "PredictRequest",
    "PredictResponse",
    "CreateDatasetRequest",
    "CreateModelRequest",
    "DeleteDatasetRequest",
    "DeleteModelRequest",
    "DeployModelRequest",
    "ExportDataRequest",
    "ExportModelRequest",
    "GetAnnotationSpecRequest",
    "GetDatasetRequest",
    "GetModelEvaluationRequest",
    "GetModelRequest",
    "ImportDataRequest",
    "ListDatasetsRequest",
    "ListDatasetsResponse",
    "ListModelEvaluationsRequest",
    "ListModelEvaluationsResponse",
    "ListModelsRequest",
    "ListModelsResponse",
    "UndeployModelRequest",
    "UpdateDatasetRequest",
    "UpdateModelRequest",
    "TextClassificationDatasetMetadata",
    "TextClassificationModelMetadata",
    "TextExtractionDatasetMetadata",
    "TextExtractionModelMetadata",
    "TextSentimentDatasetMetadata",
    "TextSentimentModelMetadata",
    "TextExtractionAnnotation",
    "TextExtractionEvaluationMetrics",
    "TextSegment",
    "TextSentimentAnnotation",
    "TextSentimentEvaluationMetrics",
    "TranslationAnnotation",
    "TranslationDatasetMetadata",
    "TranslationEvaluationMetrics",
    "TranslationModelMetadata",
)



# -*- coding: utf-8 -*-
# Copyright 2024 Google LLC
#
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
#     http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.
#
from __future__ import annotations

from typing import MutableMapping, MutableSequence

import proto  # type: ignore

from google.cloud.automl_v1.types import annotation_payload, data_items, io

__protobuf__ = proto.module(
    package="google.cloud.automl.v1",
    manifest={
        "PredictRequest",
        "PredictResponse",
        "BatchPredictRequest",
        "BatchPredictResult",
    },
)


class PredictRequest(proto.Message):
    r"""Request message for
    [PredictionService.Predict][google.cloud.automl.v1.PredictionService.Predict].

    Attributes:
        name (str):
            Required. Name of the model requested to
            serve the prediction.
        payload (google.cloud.automl_v1.types.ExamplePayload):
            Required. Payload to perform a prediction on.
            The payload must match the problem type that the
            model was trained to solve.
        params (MutableMapping[str, str]):
            Additional domain-specific parameters, any string must be up
            to 25000 characters long.

            AutoML Vision Classification

            ``score_threshold`` : (float) A value from 0.0 to 1.0. When
            the model makes predictions for an image, it will only
            produce results that have at least this confidence score.
            The default is 0.5.

            AutoML Vision Object Detection

            ``score_threshold`` : (float) When Model detects objects on
            the image, it will only produce bounding boxes which have at
            least this confidence score. Value in 0 to 1 range, default
            is 0.5.

            ``max_bounding_box_count`` : (int64) The maximum number of
            bounding boxes returned. The default is 100. The number of
            returned bounding boxes might be limited by the server.

            AutoML Tables

            ``feature_importance`` : (boolean) Whether
            [feature_importance][google.cloud.automl.v1.TablesModelColumnInfo.feature_importance]
            is populated in the returned list of
            [TablesAnnotation][google.cloud.automl.v1.TablesAnnotation]
            objects. The default is false.
    """

    name: str = proto.Field(
        proto.STRING,
        number=1,
    )
    payload: data_items.ExamplePayload = proto.Field(
        proto.MESSAGE,
        number=2,
        message=data_items.ExamplePayload,
    )
    params: MutableMapping[str, str] = proto.MapField(
        proto.STRING,
        proto.STRING,
        number=3,
    )


class PredictResponse(proto.Message):
    r"""Response message for
    [PredictionService.Predict][google.cloud.automl.v1.PredictionService.Predict].

    Attributes:
        payload (MutableSequence[google.cloud.automl_v1.types.AnnotationPayload]):
            Prediction result.
            AutoML Translation and AutoML Natural Language
            Sentiment Analysis return precisely one payload.
        preprocessed_input (google.cloud.automl_v1.types.ExamplePayload):
            The preprocessed example that AutoML actually makes
            prediction on. Empty if AutoML does not preprocess the input
            example.

            For AutoML Natural Language (Classification, Entity
            Extraction, and Sentiment Analysis), if the input is a
            document, the recognized text is returned in the
            [document_text][google.cloud.automl.v1.Document.document_text]
            property.
        metadata (MutableMapping[str, str]):
            Additional domain-specific prediction response metadata.

            AutoML Vision Object Detection

            ``max_bounding_box_count`` : (int64) The maximum number of
            bounding boxes to return per image.

            AutoML Natural Language Sentiment Analysis

            ``sentiment_score`` : (float, deprecated) A value between -1
            and 1, -1 maps to least positive sentiment, while 1 maps to
            the most positive one and the higher the score, the more
            positive the sentiment in the document is. Yet these values
            are relative to the training data, so e.g. if all data was
            positive then -1 is also positive (though the least).
            ``sentiment_score`` is not the same as "score" and
            "magnitude" from Sentiment Analysis in the Natural Language
            API.
    """

    payload: MutableSequence[
        annotation_payload.AnnotationPayload
    ] = proto.RepeatedField(
        proto.MESSAGE,
        number=1,
        message=annotation_payload.AnnotationPayload,
    )
    preprocessed_input: data_items.ExamplePayload = proto.Field(
        proto.MESSAGE,
        number=3,
        message=data_items.ExamplePayload,
    )
    metadata: MutableMapping[str, str] = proto.MapField(
        proto.STRING,
        proto.STRING,
        number=2,
    )


class BatchPredictRequest(proto.Message):
    r"""Request message for
    [PredictionService.BatchPredict][google.cloud.automl.v1.PredictionService.BatchPredict].

    Attributes:
        name (str):
            Required. Name of the model requested to
            serve the batch prediction.
        input_config (google.cloud.automl_v1.types.BatchPredictInputConfig):
            Required. The input configuration for batch
            prediction.
        output_config (google.cloud.automl_v1.types.BatchPredictOutputConfig):
            Required. The Configuration specifying where
            output predictions should be written.
        params (MutableMapping[str, str]):
            Additional domain-specific parameters for the predictions,
            any string must be up to 25000 characters long.

            AutoML Natural Language Classification

            ``score_threshold`` : (float) A value from 0.0 to 1.0. When
            the model makes predictions for a text snippet, it will only
            produce results that have at least this confidence score.
            The default is 0.5.

            AutoML Vision Classification

            ``score_threshold`` : (float) A value from 0.0 to 1.0. When
            the model makes predictions for an image, it will only
            produce results that have at least this confidence score.
            The default is 0.5.

            AutoML Vision Object Detection

            ``score_threshold`` : (float) When Model detects objects on
            the image, it will only produce bounding boxes which have at
            least this confidence score. Value in 0 to 1 range, default
            is 0.5.

            ``max_bounding_box_count`` : (int64) The maximum number of
            bounding boxes returned per image. The default is 100, the
            number of bounding boxes returned might be limited by the
            server. AutoML Video Intelligence Classification

            ``score_threshold`` : (float) A value from 0.0 to 1.0. When
            the model makes predictions for a video, it will only
            produce results that have at least this confidence score.
            The default is 0.5.

            ``segment_classification`` : (boolean) Set to true to
            request segment-level classification. AutoML Video
            Intelligence returns labels and their confidence scores for
            the entire segment of the video that user specified in the
            request configuration. The default is true.

            ``shot_classification`` : (boolean) Set to true to request
            shot-level classification. AutoML Video Intelligence
            determines the boundaries for each camera shot in the entire
            segment of the video that user specified in the request
            configuration. AutoML Video Intelligence then returns labels
            and their confidence scores for each detected shot, along
            with the start and end time of the shot. The default is
            false.

            WARNING: Model evaluation is not done for this
            classification type, the quality of it depends on training
            data, but there are no metrics provided to describe that
            quality.

            ``1s_interval_classification`` : (boolean) Set to true to
            request classification for a video at one-second intervals.
            AutoML Video Intelligence returns labels and their
            confidence scores for each second of the entire segment of
            the video that user specified in the request configuration.
            The default is false.

            WARNING: Model evaluation is not done for this
            classification type, the quality of it depends on training
            data, but there are no metrics provided to describe that
            quality.

            AutoML Video Intelligence Object Tracking

            ``score_threshold`` : (float) When Model detects objects on
            video frames, it will only produce bounding boxes which have
            at least this confidence score. Value in 0 to 1 range,
            default is 0.5.

            ``max_bounding_box_count`` : (int64) The maximum number of
            bounding boxes returned per image. The default is 100, the
            number of bounding boxes returned might be limited by the
            server.

            ``min_bounding_box_size`` : (float) Only bounding boxes with
            shortest edge at least that long as a relative value of
            video frame size are returned. Value in 0 to 1 range.
            Default is 0.
    """

    name: str = proto.Field(
        proto.STRING,
        number=1,
    )
    input_config: io.BatchPredictInputConfig = proto.Field(
        proto.MESSAGE,
        number=3,
        message=io.BatchPredictInputConfig,
    )
    output_config: io.BatchPredictOutputConfig = proto.Field(
        proto.MESSAGE,
        number=4,
        message=io.BatchPredictOutputConfig,
    )
    params: MutableMapping[str, str] = proto.MapField(
        proto.STRING,
        proto.STRING,
        number=5,
    )


class BatchPredictResult(proto.Message):
    r"""Result of the Batch Predict. This message is returned in
    [response][google.longrunning.Operation.response] of the operation
    returned by the
    [PredictionService.BatchPredict][google.cloud.automl.v1.PredictionService.BatchPredict].

    Attributes:
        metadata (MutableMapping[str, str]):
            Additional domain-specific prediction response metadata.

            AutoML Vision Object Detection

            ``max_bounding_box_count`` : (int64) The maximum number of
            bounding boxes returned per image.

            AutoML Video Intelligence Object Tracking

            ``max_bounding_box_count`` : (int64) The maximum number of
            bounding boxes returned per frame.
    """

    metadata: MutableMapping[str, str] = proto.MapField(
        proto.STRING,
        proto.STRING,
        number=1,
    )


__all__ = tuple(sorted(__protobuf__.manifest))
